"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
from documenso_sdk.models import DocumensoError
from documenso_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from enum import Enum
import httpx
import pydantic
from pydantic import model_serializer
from typing import Any, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


class TemplateCreateTemplateTemporaryVisibilityRequest(str, Enum):
    EVERYONE = "EVERYONE"
    MANAGER_AND_ABOVE = "MANAGER_AND_ABOVE"
    ADMIN = "ADMIN"


class TemplateCreateTemplateTemporaryGlobalAccessAuthRequest(str, Enum):
    ACCOUNT = "ACCOUNT"
    TWO_FACTOR_AUTH = "TWO_FACTOR_AUTH"


class TemplateCreateTemplateTemporaryGlobalActionAuthRequest(str, Enum):
    ACCOUNT = "ACCOUNT"
    PASSKEY = "PASSKEY"
    TWO_FACTOR_AUTH = "TWO_FACTOR_AUTH"
    PASSWORD = "PASSWORD"


class TemplateCreateTemplateTemporaryTypeRequest(str, Enum):
    PUBLIC = "PUBLIC"
    PRIVATE = "PRIVATE"


class TemplateCreateTemplateTemporaryDateFormat(str, Enum):
    YYYY_M_MDD_HH_MM_A = "yyyy-MM-dd hh:mm a"
    YYYY_M_MDD = "yyyy-MM-dd"
    DD_MM_SLASH_YYYY = "dd/MM/yyyy"
    MM_DD_SLASH_YYYY = "MM/dd/yyyy"
    YY_M_MDD = "yy-MM-dd"
    MMMM_DD_COMMA_YYYY = "MMMM dd, yyyy"
    EEEE_MMMM_DD_COMMA_YYYY = "EEEE, MMMM dd, yyyy"
    DD_MM_SLASH_YYYY_HH_MM_A = "dd/MM/yyyy hh:mm a"
    DD_MM_SLASH_YYYY_H_HMM = "dd/MM/yyyy HH:mm"
    MM_DD_SLASH_YYYY_HH_MM_A = "MM/dd/yyyy hh:mm a"
    MM_DD_SLASH_YYYY_H_HMM = "MM/dd/yyyy HH:mm"
    DD_DOT_MM_DOT_YYYY = "dd.MM.yyyy"
    DD_DOT_MM_DOT_YYYY_H_HMM = "dd.MM.yyyy HH:mm"
    YYYY_M_MDD_H_HMM = "yyyy-MM-dd HH:mm"
    YY_M_MDD_HH_MM_A = "yy-MM-dd hh:mm a"
    YY_M_MDD_H_HMM = "yy-MM-dd HH:mm"
    YYYY_M_MDD_H_HMMSS = "yyyy-MM-dd HH:mm:ss"
    MMMM_DD_COMMA_YYYY_HH_MM_A = "MMMM dd, yyyy hh:mm a"
    MMMM_DD_COMMA_YYYY_H_HMM = "MMMM dd, yyyy HH:mm"
    EEEE_MMMM_DD_COMMA_YYYY_HH_MM_A = "EEEE, MMMM dd, yyyy hh:mm a"
    EEEE_MMMM_DD_COMMA_YYYY_H_HMM = "EEEE, MMMM dd, yyyy HH:mm"
    ISO8601_FULL = "yyyy-MM-dd'T'HH:mm:ss.SSSXXX"


class TemplateCreateTemplateTemporaryMetaDistributionMethod(str, Enum):
    EMAIL = "EMAIL"
    NONE = "NONE"


class TemplateCreateTemplateTemporaryMetaEmailSettingsTypedDict(TypedDict):
    recipient_signing_request: NotRequired[bool]
    recipient_removed: NotRequired[bool]
    recipient_signed: NotRequired[bool]
    document_pending: NotRequired[bool]
    document_completed: NotRequired[bool]
    document_deleted: NotRequired[bool]
    owner_document_completed: NotRequired[bool]


class TemplateCreateTemplateTemporaryMetaEmailSettings(BaseModel):
    recipient_signing_request: Annotated[
        Optional[bool], pydantic.Field(alias="recipientSigningRequest")
    ] = True

    recipient_removed: Annotated[
        Optional[bool], pydantic.Field(alias="recipientRemoved")
    ] = True

    recipient_signed: Annotated[
        Optional[bool], pydantic.Field(alias="recipientSigned")
    ] = True

    document_pending: Annotated[
        Optional[bool], pydantic.Field(alias="documentPending")
    ] = True

    document_completed: Annotated[
        Optional[bool], pydantic.Field(alias="documentCompleted")
    ] = True

    document_deleted: Annotated[
        Optional[bool], pydantic.Field(alias="documentDeleted")
    ] = True

    owner_document_completed: Annotated[
        Optional[bool], pydantic.Field(alias="ownerDocumentCompleted")
    ] = True


class TemplateCreateTemplateTemporaryLanguage(str, Enum):
    DE = "de"
    EN = "en"
    FR = "fr"
    ES = "es"
    IT = "it"
    PL = "pl"


class TemplateCreateTemplateTemporarySigningOrderRequest(str, Enum):
    PARALLEL = "PARALLEL"
    SEQUENTIAL = "SEQUENTIAL"


class TemplateCreateTemplateTemporaryMetaTypedDict(TypedDict):
    subject: NotRequired[str]
    message: NotRequired[str]
    timezone: NotRequired[str]
    date_format: NotRequired[TemplateCreateTemplateTemporaryDateFormat]
    distribution_method: NotRequired[
        TemplateCreateTemplateTemporaryMetaDistributionMethod
    ]
    email_id: NotRequired[Nullable[str]]
    email_reply_to: NotRequired[Nullable[str]]
    email_settings: NotRequired[
        TemplateCreateTemplateTemporaryMetaEmailSettingsTypedDict
    ]
    redirect_url: NotRequired[str]
    language: NotRequired[TemplateCreateTemplateTemporaryLanguage]
    typed_signature_enabled: NotRequired[bool]
    upload_signature_enabled: NotRequired[bool]
    draw_signature_enabled: NotRequired[bool]
    signing_order: NotRequired[TemplateCreateTemplateTemporarySigningOrderRequest]
    allow_dictate_next_signer: NotRequired[bool]


class TemplateCreateTemplateTemporaryMeta(BaseModel):
    subject: Optional[str] = None

    message: Optional[str] = None

    timezone: Optional[str] = None

    date_format: Annotated[
        Optional[TemplateCreateTemplateTemporaryDateFormat],
        pydantic.Field(alias="dateFormat"),
    ] = None

    distribution_method: Annotated[
        Optional[TemplateCreateTemplateTemporaryMetaDistributionMethod],
        pydantic.Field(alias="distributionMethod"),
    ] = None

    email_id: Annotated[OptionalNullable[str], pydantic.Field(alias="emailId")] = UNSET

    email_reply_to: Annotated[
        OptionalNullable[str], pydantic.Field(alias="emailReplyTo")
    ] = UNSET

    email_settings: Annotated[
        Optional[TemplateCreateTemplateTemporaryMetaEmailSettings],
        pydantic.Field(alias="emailSettings"),
    ] = None

    redirect_url: Annotated[Optional[str], pydantic.Field(alias="redirectUrl")] = None

    language: Optional[TemplateCreateTemplateTemporaryLanguage] = None

    typed_signature_enabled: Annotated[
        Optional[bool], pydantic.Field(alias="typedSignatureEnabled")
    ] = None

    upload_signature_enabled: Annotated[
        Optional[bool], pydantic.Field(alias="uploadSignatureEnabled")
    ] = None

    draw_signature_enabled: Annotated[
        Optional[bool], pydantic.Field(alias="drawSignatureEnabled")
    ] = None

    signing_order: Annotated[
        Optional[TemplateCreateTemplateTemporarySigningOrderRequest],
        pydantic.Field(alias="signingOrder"),
    ] = None

    allow_dictate_next_signer: Annotated[
        Optional[bool], pydantic.Field(alias="allowDictateNextSigner")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "subject",
            "message",
            "timezone",
            "dateFormat",
            "distributionMethod",
            "emailId",
            "emailReplyTo",
            "emailSettings",
            "redirectUrl",
            "language",
            "typedSignatureEnabled",
            "uploadSignatureEnabled",
            "drawSignatureEnabled",
            "signingOrder",
            "allowDictateNextSigner",
        ]
        nullable_fields = ["emailId", "emailReplyTo"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class TemplateCreateTemplateTemporaryTypeLink(str, Enum):
    LINK = "link"


class TemplateCreateTemplateTemporaryAttachmentTypedDict(TypedDict):
    label: str
    data: str
    type: NotRequired[TemplateCreateTemplateTemporaryTypeLink]


class TemplateCreateTemplateTemporaryAttachment(BaseModel):
    label: str

    data: str

    type: Optional[TemplateCreateTemplateTemporaryTypeLink] = (
        TemplateCreateTemplateTemporaryTypeLink.LINK
    )


class TemplateCreateTemplateTemporaryRequestTypedDict(TypedDict):
    title: str
    folder_id: NotRequired[str]
    external_id: NotRequired[Nullable[str]]
    visibility: NotRequired[TemplateCreateTemplateTemporaryVisibilityRequest]
    global_access_auth: NotRequired[
        List[TemplateCreateTemplateTemporaryGlobalAccessAuthRequest]
    ]
    global_action_auth: NotRequired[
        List[TemplateCreateTemplateTemporaryGlobalActionAuthRequest]
    ]
    public_title: NotRequired[str]
    public_description: NotRequired[str]
    type: NotRequired[TemplateCreateTemplateTemporaryTypeRequest]
    meta: NotRequired[TemplateCreateTemplateTemporaryMetaTypedDict]
    attachments: NotRequired[List[TemplateCreateTemplateTemporaryAttachmentTypedDict]]


class TemplateCreateTemplateTemporaryRequest(BaseModel):
    title: str

    folder_id: Annotated[Optional[str], pydantic.Field(alias="folderId")] = None

    external_id: Annotated[
        OptionalNullable[str], pydantic.Field(alias="externalId")
    ] = UNSET

    visibility: Optional[TemplateCreateTemplateTemporaryVisibilityRequest] = None

    global_access_auth: Annotated[
        Optional[List[TemplateCreateTemplateTemporaryGlobalAccessAuthRequest]],
        pydantic.Field(alias="globalAccessAuth"),
    ] = None

    global_action_auth: Annotated[
        Optional[List[TemplateCreateTemplateTemporaryGlobalActionAuthRequest]],
        pydantic.Field(alias="globalActionAuth"),
    ] = None

    public_title: Annotated[Optional[str], pydantic.Field(alias="publicTitle")] = None

    public_description: Annotated[
        Optional[str], pydantic.Field(alias="publicDescription")
    ] = None

    type: Optional[TemplateCreateTemplateTemporaryTypeRequest] = None

    meta: Optional[TemplateCreateTemplateTemporaryMeta] = None

    attachments: Optional[List[TemplateCreateTemplateTemporaryAttachment]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "folderId",
            "externalId",
            "visibility",
            "globalAccessAuth",
            "globalActionAuth",
            "publicTitle",
            "publicDescription",
            "type",
            "meta",
            "attachments",
        ]
        nullable_fields = ["externalId"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class TemplateCreateTemplateTemporaryInternalServerErrorIssueTypedDict(TypedDict):
    message: str


class TemplateCreateTemplateTemporaryInternalServerErrorIssue(BaseModel):
    message: str


class TemplateCreateTemplateTemporaryInternalServerErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[TemplateCreateTemplateTemporaryInternalServerErrorIssue]] = (
        None
    )


@dataclass(unsafe_hash=True)
class TemplateCreateTemplateTemporaryInternalServerError(DocumensoError):
    r"""Internal server error"""

    data: TemplateCreateTemplateTemporaryInternalServerErrorData = field(hash=False)

    def __init__(
        self,
        data: TemplateCreateTemplateTemporaryInternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class TemplateCreateTemplateTemporaryForbiddenIssueTypedDict(TypedDict):
    message: str


class TemplateCreateTemplateTemporaryForbiddenIssue(BaseModel):
    message: str


class TemplateCreateTemplateTemporaryForbiddenErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[TemplateCreateTemplateTemporaryForbiddenIssue]] = None


@dataclass(unsafe_hash=True)
class TemplateCreateTemplateTemporaryForbiddenError(DocumensoError):
    r"""Insufficient access"""

    data: TemplateCreateTemplateTemporaryForbiddenErrorData = field(hash=False)

    def __init__(
        self,
        data: TemplateCreateTemplateTemporaryForbiddenErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class TemplateCreateTemplateTemporaryUnauthorizedIssueTypedDict(TypedDict):
    message: str


class TemplateCreateTemplateTemporaryUnauthorizedIssue(BaseModel):
    message: str


class TemplateCreateTemplateTemporaryUnauthorizedErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[TemplateCreateTemplateTemporaryUnauthorizedIssue]] = None


@dataclass(unsafe_hash=True)
class TemplateCreateTemplateTemporaryUnauthorizedError(DocumensoError):
    r"""Authorization not provided"""

    data: TemplateCreateTemplateTemporaryUnauthorizedErrorData = field(hash=False)

    def __init__(
        self,
        data: TemplateCreateTemplateTemporaryUnauthorizedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class TemplateCreateTemplateTemporaryBadRequestIssueTypedDict(TypedDict):
    message: str


class TemplateCreateTemplateTemporaryBadRequestIssue(BaseModel):
    message: str


class TemplateCreateTemplateTemporaryBadRequestErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[TemplateCreateTemplateTemporaryBadRequestIssue]] = None


@dataclass(unsafe_hash=True)
class TemplateCreateTemplateTemporaryBadRequestError(DocumensoError):
    r"""Invalid input data"""

    data: TemplateCreateTemplateTemporaryBadRequestErrorData = field(hash=False)

    def __init__(
        self,
        data: TemplateCreateTemplateTemporaryBadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class TemplateCreateTemplateTemporaryTypeResponse(str, Enum):
    PUBLIC = "PUBLIC"
    PRIVATE = "PRIVATE"


class TemplateCreateTemplateTemporaryVisibilityResponse(str, Enum):
    EVERYONE = "EVERYONE"
    MANAGER_AND_ABOVE = "MANAGER_AND_ABOVE"
    ADMIN = "ADMIN"


class TemplateCreateTemplateTemporaryGlobalAccessAuthResponse(str, Enum):
    ACCOUNT = "ACCOUNT"
    TWO_FACTOR_AUTH = "TWO_FACTOR_AUTH"


class TemplateCreateTemplateTemporaryGlobalActionAuthResponse(str, Enum):
    ACCOUNT = "ACCOUNT"
    PASSKEY = "PASSKEY"
    TWO_FACTOR_AUTH = "TWO_FACTOR_AUTH"
    PASSWORD = "PASSWORD"


class TemplateCreateTemplateTemporaryAuthOptionsTypedDict(TypedDict):
    global_access_auth: List[TemplateCreateTemplateTemporaryGlobalAccessAuthResponse]
    global_action_auth: List[TemplateCreateTemplateTemporaryGlobalActionAuthResponse]


class TemplateCreateTemplateTemporaryAuthOptions(BaseModel):
    global_access_auth: Annotated[
        List[TemplateCreateTemplateTemporaryGlobalAccessAuthResponse],
        pydantic.Field(alias="globalAccessAuth"),
    ]

    global_action_auth: Annotated[
        List[TemplateCreateTemplateTemporaryGlobalActionAuthResponse],
        pydantic.Field(alias="globalActionAuth"),
    ]


class TemplateCreateTemplateTemporaryTemplateDocumentDataType(str, Enum):
    S3_PATH = "S3_PATH"
    BYTES = "BYTES"
    BYTES_64 = "BYTES_64"


class TemplateCreateTemplateTemporaryTemplateDocumentDataTypedDict(TypedDict):
    type: TemplateCreateTemplateTemporaryTemplateDocumentDataType
    id: str
    data: str
    initial_data: str
    envelope_item_id: str


class TemplateCreateTemplateTemporaryTemplateDocumentData(BaseModel):
    type: TemplateCreateTemplateTemporaryTemplateDocumentDataType

    id: str

    data: str

    initial_data: Annotated[str, pydantic.Field(alias="initialData")]

    envelope_item_id: Annotated[str, pydantic.Field(alias="envelopeItemId")]


class TemplateCreateTemplateTemporaryTemplateMetaSigningOrder(str, Enum):
    PARALLEL = "PARALLEL"
    SEQUENTIAL = "SEQUENTIAL"


class TemplateCreateTemplateTemporaryTemplateMetaDistributionMethod(str, Enum):
    EMAIL = "EMAIL"
    NONE = "NONE"


class TemplateCreateTemplateTemporaryTemplateMetaEmailSettingsTypedDict(TypedDict):
    recipient_signing_request: NotRequired[bool]
    recipient_removed: NotRequired[bool]
    recipient_signed: NotRequired[bool]
    document_pending: NotRequired[bool]
    document_completed: NotRequired[bool]
    document_deleted: NotRequired[bool]
    owner_document_completed: NotRequired[bool]


class TemplateCreateTemplateTemporaryTemplateMetaEmailSettings(BaseModel):
    recipient_signing_request: Annotated[
        Optional[bool], pydantic.Field(alias="recipientSigningRequest")
    ] = True

    recipient_removed: Annotated[
        Optional[bool], pydantic.Field(alias="recipientRemoved")
    ] = True

    recipient_signed: Annotated[
        Optional[bool], pydantic.Field(alias="recipientSigned")
    ] = True

    document_pending: Annotated[
        Optional[bool], pydantic.Field(alias="documentPending")
    ] = True

    document_completed: Annotated[
        Optional[bool], pydantic.Field(alias="documentCompleted")
    ] = True

    document_deleted: Annotated[
        Optional[bool], pydantic.Field(alias="documentDeleted")
    ] = True

    owner_document_completed: Annotated[
        Optional[bool], pydantic.Field(alias="ownerDocumentCompleted")
    ] = True


class TemplateCreateTemplateTemporaryTemplateMetaTypedDict(TypedDict):
    id: str
    subject: Nullable[str]
    message: Nullable[str]
    timezone: Nullable[str]
    date_format: Nullable[str]
    signing_order: TemplateCreateTemplateTemporaryTemplateMetaSigningOrder
    typed_signature_enabled: bool
    upload_signature_enabled: bool
    draw_signature_enabled: bool
    allow_dictate_next_signer: bool
    distribution_method: TemplateCreateTemplateTemporaryTemplateMetaDistributionMethod
    redirect_url: Nullable[str]
    language: str
    email_settings: Nullable[
        TemplateCreateTemplateTemporaryTemplateMetaEmailSettingsTypedDict
    ]
    email_id: Nullable[str]
    email_reply_to: Nullable[str]
    template_id: Nullable[float]


class TemplateCreateTemplateTemporaryTemplateMeta(BaseModel):
    id: str

    subject: Nullable[str]

    message: Nullable[str]

    timezone: Nullable[str]

    date_format: Annotated[Nullable[str], pydantic.Field(alias="dateFormat")]

    signing_order: Annotated[
        TemplateCreateTemplateTemporaryTemplateMetaSigningOrder,
        pydantic.Field(alias="signingOrder"),
    ]

    typed_signature_enabled: Annotated[
        bool, pydantic.Field(alias="typedSignatureEnabled")
    ]

    upload_signature_enabled: Annotated[
        bool, pydantic.Field(alias="uploadSignatureEnabled")
    ]

    draw_signature_enabled: Annotated[
        bool, pydantic.Field(alias="drawSignatureEnabled")
    ]

    allow_dictate_next_signer: Annotated[
        bool, pydantic.Field(alias="allowDictateNextSigner")
    ]

    distribution_method: Annotated[
        TemplateCreateTemplateTemporaryTemplateMetaDistributionMethod,
        pydantic.Field(alias="distributionMethod"),
    ]

    redirect_url: Annotated[Nullable[str], pydantic.Field(alias="redirectUrl")]

    language: str

    email_settings: Annotated[
        Nullable[TemplateCreateTemplateTemporaryTemplateMetaEmailSettings],
        pydantic.Field(alias="emailSettings"),
    ]

    email_id: Annotated[Nullable[str], pydantic.Field(alias="emailId")]

    email_reply_to: Annotated[Nullable[str], pydantic.Field(alias="emailReplyTo")]

    template_id: Annotated[Nullable[float], pydantic.Field(alias="templateId")]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = [
            "subject",
            "message",
            "timezone",
            "dateFormat",
            "redirectUrl",
            "emailSettings",
            "emailId",
            "emailReplyTo",
            "templateId",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class TemplateCreateTemplateTemporaryDirectLinkTypedDict(TypedDict):
    id: str
    envelope_id: str
    token: str
    created_at: str
    enabled: bool
    direct_template_recipient_id: float
    template_id: float


class TemplateCreateTemplateTemporaryDirectLink(BaseModel):
    id: str

    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]

    token: str

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]

    enabled: bool

    direct_template_recipient_id: Annotated[
        float, pydantic.Field(alias="directTemplateRecipientId")
    ]

    template_id: Annotated[float, pydantic.Field(alias="templateId")]


class TemplateCreateTemplateTemporaryUserTypedDict(TypedDict):
    id: float
    name: Nullable[str]
    email: str


class TemplateCreateTemplateTemporaryUser(BaseModel):
    id: float

    name: Nullable[str]

    email: str

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["name"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class TemplateCreateTemplateTemporaryRole(str, Enum):
    CC = "CC"
    SIGNER = "SIGNER"
    VIEWER = "VIEWER"
    APPROVER = "APPROVER"
    ASSISTANT = "ASSISTANT"


class TemplateCreateTemplateTemporaryReadStatus(str, Enum):
    NOT_OPENED = "NOT_OPENED"
    OPENED = "OPENED"


class TemplateCreateTemplateTemporarySigningStatus(str, Enum):
    NOT_SIGNED = "NOT_SIGNED"
    SIGNED = "SIGNED"
    REJECTED = "REJECTED"


class TemplateCreateTemplateTemporarySendStatus(str, Enum):
    NOT_SENT = "NOT_SENT"
    SENT = "SENT"


class TemplateCreateTemplateTemporaryAccessAuth(str, Enum):
    ACCOUNT = "ACCOUNT"
    TWO_FACTOR_AUTH = "TWO_FACTOR_AUTH"


class TemplateCreateTemplateTemporaryActionAuth(str, Enum):
    ACCOUNT = "ACCOUNT"
    PASSKEY = "PASSKEY"
    TWO_FACTOR_AUTH = "TWO_FACTOR_AUTH"
    PASSWORD = "PASSWORD"
    EXPLICIT_NONE = "EXPLICIT_NONE"


class TemplateCreateTemplateTemporaryRecipientAuthOptionsTypedDict(TypedDict):
    access_auth: List[TemplateCreateTemplateTemporaryAccessAuth]
    action_auth: List[TemplateCreateTemplateTemporaryActionAuth]


class TemplateCreateTemplateTemporaryRecipientAuthOptions(BaseModel):
    access_auth: Annotated[
        List[TemplateCreateTemplateTemporaryAccessAuth],
        pydantic.Field(alias="accessAuth"),
    ]

    action_auth: Annotated[
        List[TemplateCreateTemplateTemporaryActionAuth],
        pydantic.Field(alias="actionAuth"),
    ]


class TemplateCreateTemplateTemporaryRecipientTypedDict(TypedDict):
    envelope_id: str
    role: TemplateCreateTemplateTemporaryRole
    read_status: TemplateCreateTemplateTemporaryReadStatus
    signing_status: TemplateCreateTemplateTemporarySigningStatus
    send_status: TemplateCreateTemplateTemporarySendStatus
    id: float
    email: str
    name: str
    token: str
    document_deleted_at: Nullable[str]
    expired: Nullable[str]
    signed_at: Nullable[str]
    auth_options: Nullable[TemplateCreateTemplateTemporaryRecipientAuthOptionsTypedDict]
    signing_order: Nullable[float]
    rejection_reason: Nullable[str]
    document_id: NotRequired[Nullable[float]]
    template_id: NotRequired[Nullable[float]]


class TemplateCreateTemplateTemporaryRecipient(BaseModel):
    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]

    role: TemplateCreateTemplateTemporaryRole

    read_status: Annotated[
        TemplateCreateTemplateTemporaryReadStatus, pydantic.Field(alias="readStatus")
    ]

    signing_status: Annotated[
        TemplateCreateTemplateTemporarySigningStatus,
        pydantic.Field(alias="signingStatus"),
    ]

    send_status: Annotated[
        TemplateCreateTemplateTemporarySendStatus, pydantic.Field(alias="sendStatus")
    ]

    id: float

    email: str

    name: str

    token: str

    document_deleted_at: Annotated[
        Nullable[str], pydantic.Field(alias="documentDeletedAt")
    ]

    expired: Nullable[str]

    signed_at: Annotated[Nullable[str], pydantic.Field(alias="signedAt")]

    auth_options: Annotated[
        Nullable[TemplateCreateTemplateTemporaryRecipientAuthOptions],
        pydantic.Field(alias="authOptions"),
    ]

    signing_order: Annotated[Nullable[float], pydantic.Field(alias="signingOrder")]

    rejection_reason: Annotated[Nullable[str], pydantic.Field(alias="rejectionReason")]

    document_id: Annotated[
        OptionalNullable[float], pydantic.Field(alias="documentId")
    ] = UNSET

    template_id: Annotated[
        OptionalNullable[float], pydantic.Field(alias="templateId")
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["documentId", "templateId"]
        nullable_fields = [
            "documentDeletedAt",
            "expired",
            "signedAt",
            "authOptions",
            "signingOrder",
            "rejectionReason",
            "documentId",
            "templateId",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class TemplateCreateTemplateTemporaryFieldType(str, Enum):
    SIGNATURE = "SIGNATURE"
    FREE_SIGNATURE = "FREE_SIGNATURE"
    INITIALS = "INITIALS"
    NAME = "NAME"
    EMAIL = "EMAIL"
    DATE = "DATE"
    TEXT = "TEXT"
    NUMBER = "NUMBER"
    RADIO = "RADIO"
    CHECKBOX = "CHECKBOX"
    DROPDOWN = "DROPDOWN"


class TemplateCreateTemplateTemporaryTypeDropdown(str, Enum):
    DROPDOWN = "dropdown"


class TemplateCreateTemplateTemporaryValue3TypedDict(TypedDict):
    value: str


class TemplateCreateTemplateTemporaryValue3(BaseModel):
    value: str


class TemplateCreateTemplateTemporaryFieldMetaDropdownTypedDict(TypedDict):
    type: TemplateCreateTemplateTemporaryTypeDropdown
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[TemplateCreateTemplateTemporaryValue3TypedDict]]
    default_value: NotRequired[str]


class TemplateCreateTemplateTemporaryFieldMetaDropdown(BaseModel):
    type: TemplateCreateTemplateTemporaryTypeDropdown

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[TemplateCreateTemplateTemporaryValue3]] = None

    default_value: Annotated[Optional[str], pydantic.Field(alias="defaultValue")] = None


class TemplateCreateTemplateTemporaryTypeCheckbox(str, Enum):
    CHECKBOX = "checkbox"


class TemplateCreateTemplateTemporaryValue2TypedDict(TypedDict):
    id: float
    checked: bool
    value: str


class TemplateCreateTemplateTemporaryValue2(BaseModel):
    id: float

    checked: bool

    value: str


class TemplateCreateTemplateTemporaryDirection2(str, Enum):
    VERTICAL = "vertical"
    HORIZONTAL = "horizontal"


class TemplateCreateTemplateTemporaryFieldMetaCheckboxTypedDict(TypedDict):
    type: TemplateCreateTemplateTemporaryTypeCheckbox
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[TemplateCreateTemplateTemporaryValue2TypedDict]]
    validation_rule: NotRequired[str]
    validation_length: NotRequired[float]
    direction: NotRequired[TemplateCreateTemplateTemporaryDirection2]


class TemplateCreateTemplateTemporaryFieldMetaCheckbox(BaseModel):
    type: TemplateCreateTemplateTemporaryTypeCheckbox

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[TemplateCreateTemplateTemporaryValue2]] = None

    validation_rule: Annotated[
        Optional[str], pydantic.Field(alias="validationRule")
    ] = None

    validation_length: Annotated[
        Optional[float], pydantic.Field(alias="validationLength")
    ] = None

    direction: Optional[TemplateCreateTemplateTemporaryDirection2] = (
        TemplateCreateTemplateTemporaryDirection2.VERTICAL
    )


class TemplateCreateTemplateTemporaryTypeRadio(str, Enum):
    RADIO = "radio"


class TemplateCreateTemplateTemporaryValue1TypedDict(TypedDict):
    id: float
    checked: bool
    value: str


class TemplateCreateTemplateTemporaryValue1(BaseModel):
    id: float

    checked: bool

    value: str


class TemplateCreateTemplateTemporaryDirection1(str, Enum):
    VERTICAL = "vertical"
    HORIZONTAL = "horizontal"


class TemplateCreateTemplateTemporaryFieldMetaRadioTypedDict(TypedDict):
    type: TemplateCreateTemplateTemporaryTypeRadio
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[TemplateCreateTemplateTemporaryValue1TypedDict]]
    direction: NotRequired[TemplateCreateTemplateTemporaryDirection1]


class TemplateCreateTemplateTemporaryFieldMetaRadio(BaseModel):
    type: TemplateCreateTemplateTemporaryTypeRadio

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[TemplateCreateTemplateTemporaryValue1]] = None

    direction: Optional[TemplateCreateTemplateTemporaryDirection1] = (
        TemplateCreateTemplateTemporaryDirection1.VERTICAL
    )


class TemplateCreateTemplateTemporaryTypeNumber(str, Enum):
    NUMBER = "number"


class TemplateCreateTemplateTemporaryTextAlign6(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class TemplateCreateTemplateTemporaryVerticalAlign2(str, Enum):
    TOP = "top"
    MIDDLE = "middle"
    BOTTOM = "bottom"


class TemplateCreateTemplateTemporaryFieldMetaNumberTypedDict(TypedDict):
    type: TemplateCreateTemplateTemporaryTypeNumber
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    number_format: NotRequired[Nullable[str]]
    value: NotRequired[str]
    min_value: NotRequired[Nullable[float]]
    max_value: NotRequired[Nullable[float]]
    text_align: NotRequired[TemplateCreateTemplateTemporaryTextAlign6]
    line_height: NotRequired[Nullable[float]]
    letter_spacing: NotRequired[Nullable[float]]
    vertical_align: NotRequired[Nullable[TemplateCreateTemplateTemporaryVerticalAlign2]]


class TemplateCreateTemplateTemporaryFieldMetaNumber(BaseModel):
    type: TemplateCreateTemplateTemporaryTypeNumber

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    number_format: Annotated[
        OptionalNullable[str], pydantic.Field(alias="numberFormat")
    ] = UNSET

    value: Optional[str] = None

    min_value: Annotated[OptionalNullable[float], pydantic.Field(alias="minValue")] = (
        UNSET
    )

    max_value: Annotated[OptionalNullable[float], pydantic.Field(alias="maxValue")] = (
        UNSET
    )

    text_align: Annotated[
        Optional[TemplateCreateTemplateTemporaryTextAlign6],
        pydantic.Field(alias="textAlign"),
    ] = None

    line_height: Annotated[
        OptionalNullable[float], pydantic.Field(alias="lineHeight")
    ] = UNSET

    letter_spacing: Annotated[
        OptionalNullable[float], pydantic.Field(alias="letterSpacing")
    ] = UNSET

    vertical_align: Annotated[
        OptionalNullable[TemplateCreateTemplateTemporaryVerticalAlign2],
        pydantic.Field(alias="verticalAlign"),
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "label",
            "placeholder",
            "required",
            "readOnly",
            "fontSize",
            "numberFormat",
            "value",
            "minValue",
            "maxValue",
            "textAlign",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        nullable_fields = [
            "numberFormat",
            "minValue",
            "maxValue",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class TemplateCreateTemplateTemporaryTypeText(str, Enum):
    TEXT = "text"


class TemplateCreateTemplateTemporaryTextAlign5(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class TemplateCreateTemplateTemporaryVerticalAlign1(str, Enum):
    TOP = "top"
    MIDDLE = "middle"
    BOTTOM = "bottom"


class TemplateCreateTemplateTemporaryFieldMetaTextTypedDict(TypedDict):
    type: TemplateCreateTemplateTemporaryTypeText
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text: NotRequired[str]
    character_limit: NotRequired[float]
    text_align: NotRequired[TemplateCreateTemplateTemporaryTextAlign5]
    line_height: NotRequired[Nullable[float]]
    letter_spacing: NotRequired[Nullable[float]]
    vertical_align: NotRequired[Nullable[TemplateCreateTemplateTemporaryVerticalAlign1]]


class TemplateCreateTemplateTemporaryFieldMetaText(BaseModel):
    type: TemplateCreateTemplateTemporaryTypeText

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text: Optional[str] = None

    character_limit: Annotated[
        Optional[float], pydantic.Field(alias="characterLimit")
    ] = None

    text_align: Annotated[
        Optional[TemplateCreateTemplateTemporaryTextAlign5],
        pydantic.Field(alias="textAlign"),
    ] = None

    line_height: Annotated[
        OptionalNullable[float], pydantic.Field(alias="lineHeight")
    ] = UNSET

    letter_spacing: Annotated[
        OptionalNullable[float], pydantic.Field(alias="letterSpacing")
    ] = UNSET

    vertical_align: Annotated[
        OptionalNullable[TemplateCreateTemplateTemporaryVerticalAlign1],
        pydantic.Field(alias="verticalAlign"),
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "label",
            "placeholder",
            "required",
            "readOnly",
            "fontSize",
            "text",
            "characterLimit",
            "textAlign",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        nullable_fields = ["lineHeight", "letterSpacing", "verticalAlign"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class TemplateCreateTemplateTemporaryTypeDate(str, Enum):
    DATE = "date"


class TemplateCreateTemplateTemporaryTextAlign4(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class TemplateCreateTemplateTemporaryFieldMetaDateTypedDict(TypedDict):
    type: TemplateCreateTemplateTemporaryTypeDate
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[TemplateCreateTemplateTemporaryTextAlign4]


class TemplateCreateTemplateTemporaryFieldMetaDate(BaseModel):
    type: TemplateCreateTemplateTemporaryTypeDate

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[TemplateCreateTemplateTemporaryTextAlign4],
        pydantic.Field(alias="textAlign"),
    ] = None


class TemplateCreateTemplateTemporaryTypeEmail(str, Enum):
    EMAIL = "email"


class TemplateCreateTemplateTemporaryTextAlign3(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class TemplateCreateTemplateTemporaryFieldMetaEmailTypedDict(TypedDict):
    type: TemplateCreateTemplateTemporaryTypeEmail
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[TemplateCreateTemplateTemporaryTextAlign3]


class TemplateCreateTemplateTemporaryFieldMetaEmail(BaseModel):
    type: TemplateCreateTemplateTemporaryTypeEmail

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[TemplateCreateTemplateTemporaryTextAlign3],
        pydantic.Field(alias="textAlign"),
    ] = None


class TemplateCreateTemplateTemporaryTypeName(str, Enum):
    NAME = "name"


class TemplateCreateTemplateTemporaryTextAlign2(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class TemplateCreateTemplateTemporaryFieldMetaNameTypedDict(TypedDict):
    type: TemplateCreateTemplateTemporaryTypeName
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[TemplateCreateTemplateTemporaryTextAlign2]


class TemplateCreateTemplateTemporaryFieldMetaName(BaseModel):
    type: TemplateCreateTemplateTemporaryTypeName

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[TemplateCreateTemplateTemporaryTextAlign2],
        pydantic.Field(alias="textAlign"),
    ] = None


class TemplateCreateTemplateTemporaryTypeInitials(str, Enum):
    INITIALS = "initials"


class TemplateCreateTemplateTemporaryTextAlign1(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class TemplateCreateTemplateTemporaryFieldMetaInitialsTypedDict(TypedDict):
    type: TemplateCreateTemplateTemporaryTypeInitials
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[TemplateCreateTemplateTemporaryTextAlign1]


class TemplateCreateTemplateTemporaryFieldMetaInitials(BaseModel):
    type: TemplateCreateTemplateTemporaryTypeInitials

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[TemplateCreateTemplateTemporaryTextAlign1],
        pydantic.Field(alias="textAlign"),
    ] = None


class TemplateCreateTemplateTemporaryTypeSignature(str, Enum):
    SIGNATURE = "signature"


class TemplateCreateTemplateTemporaryFieldMetaSignatureTypedDict(TypedDict):
    type: TemplateCreateTemplateTemporaryTypeSignature
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]


class TemplateCreateTemplateTemporaryFieldMetaSignature(BaseModel):
    type: TemplateCreateTemplateTemporaryTypeSignature

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12


TemplateCreateTemplateTemporaryFieldMetaUnionTypedDict = TypeAliasType(
    "TemplateCreateTemplateTemporaryFieldMetaUnionTypedDict",
    Union[
        TemplateCreateTemplateTemporaryFieldMetaSignatureTypedDict,
        TemplateCreateTemplateTemporaryFieldMetaInitialsTypedDict,
        TemplateCreateTemplateTemporaryFieldMetaNameTypedDict,
        TemplateCreateTemplateTemporaryFieldMetaEmailTypedDict,
        TemplateCreateTemplateTemporaryFieldMetaDateTypedDict,
        TemplateCreateTemplateTemporaryFieldMetaRadioTypedDict,
        TemplateCreateTemplateTemporaryFieldMetaDropdownTypedDict,
        TemplateCreateTemplateTemporaryFieldMetaCheckboxTypedDict,
        TemplateCreateTemplateTemporaryFieldMetaTextTypedDict,
        TemplateCreateTemplateTemporaryFieldMetaNumberTypedDict,
    ],
)


TemplateCreateTemplateTemporaryFieldMetaUnion = TypeAliasType(
    "TemplateCreateTemplateTemporaryFieldMetaUnion",
    Union[
        TemplateCreateTemplateTemporaryFieldMetaSignature,
        TemplateCreateTemplateTemporaryFieldMetaInitials,
        TemplateCreateTemplateTemporaryFieldMetaName,
        TemplateCreateTemplateTemporaryFieldMetaEmail,
        TemplateCreateTemplateTemporaryFieldMetaDate,
        TemplateCreateTemplateTemporaryFieldMetaRadio,
        TemplateCreateTemplateTemporaryFieldMetaDropdown,
        TemplateCreateTemplateTemporaryFieldMetaCheckbox,
        TemplateCreateTemplateTemporaryFieldMetaText,
        TemplateCreateTemplateTemporaryFieldMetaNumber,
    ],
)


class TemplateCreateTemplateTemporaryFieldTypedDict(TypedDict):
    envelope_id: str
    envelope_item_id: str
    type: TemplateCreateTemplateTemporaryFieldType
    id: float
    secondary_id: str
    recipient_id: float
    page: float
    position_x: Any
    position_y: Any
    width: Any
    height: Any
    custom_text: str
    inserted: bool
    field_meta: Nullable[TemplateCreateTemplateTemporaryFieldMetaUnionTypedDict]
    document_id: NotRequired[Nullable[float]]
    template_id: NotRequired[Nullable[float]]


class TemplateCreateTemplateTemporaryField(BaseModel):
    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]

    envelope_item_id: Annotated[str, pydantic.Field(alias="envelopeItemId")]

    type: TemplateCreateTemplateTemporaryFieldType

    id: float

    secondary_id: Annotated[str, pydantic.Field(alias="secondaryId")]

    recipient_id: Annotated[float, pydantic.Field(alias="recipientId")]

    page: float

    position_x: Annotated[Any, pydantic.Field(alias="positionX")]

    position_y: Annotated[Any, pydantic.Field(alias="positionY")]

    width: Any

    height: Any

    custom_text: Annotated[str, pydantic.Field(alias="customText")]

    inserted: bool

    field_meta: Annotated[
        Nullable[TemplateCreateTemplateTemporaryFieldMetaUnion],
        pydantic.Field(alias="fieldMeta"),
    ]

    document_id: Annotated[
        OptionalNullable[float], pydantic.Field(alias="documentId")
    ] = UNSET

    template_id: Annotated[
        OptionalNullable[float], pydantic.Field(alias="templateId")
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["documentId", "templateId"]
        nullable_fields = ["fieldMeta", "documentId", "templateId"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class TemplateCreateTemplateTemporaryFolderType(str, Enum):
    DOCUMENT = "DOCUMENT"
    TEMPLATE = "TEMPLATE"


class TemplateCreateTemplateTemporaryFolderVisibility(str, Enum):
    EVERYONE = "EVERYONE"
    MANAGER_AND_ABOVE = "MANAGER_AND_ABOVE"
    ADMIN = "ADMIN"


class TemplateCreateTemplateTemporaryFolderTypedDict(TypedDict):
    id: str
    name: str
    type: TemplateCreateTemplateTemporaryFolderType
    visibility: TemplateCreateTemplateTemporaryFolderVisibility
    user_id: float
    team_id: float
    pinned: bool
    parent_id: Nullable[str]
    created_at: str
    updated_at: str


class TemplateCreateTemplateTemporaryFolder(BaseModel):
    id: str

    name: str

    type: TemplateCreateTemplateTemporaryFolderType

    visibility: TemplateCreateTemplateTemporaryFolderVisibility

    user_id: Annotated[float, pydantic.Field(alias="userId")]

    team_id: Annotated[float, pydantic.Field(alias="teamId")]

    pinned: bool

    parent_id: Annotated[Nullable[str], pydantic.Field(alias="parentId")]

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]

    updated_at: Annotated[str, pydantic.Field(alias="updatedAt")]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["parentId"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class TemplateCreateTemplateTemporaryEnvelopeItemTypedDict(TypedDict):
    id: str
    envelope_id: str


class TemplateCreateTemplateTemporaryEnvelopeItem(BaseModel):
    id: str

    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]


class TemplateCreateTemplateTemporaryTemplateTypedDict(TypedDict):
    type: TemplateCreateTemplateTemporaryTypeResponse
    visibility: TemplateCreateTemplateTemporaryVisibilityResponse
    id: float
    external_id: Nullable[str]
    title: str
    user_id: float
    team_id: float
    auth_options: Nullable[TemplateCreateTemplateTemporaryAuthOptionsTypedDict]
    created_at: str
    updated_at: str
    public_title: str
    public_description: str
    folder_id: Nullable[str]
    envelope_id: str
    template_document_data: TemplateCreateTemplateTemporaryTemplateDocumentDataTypedDict
    template_meta: TemplateCreateTemplateTemporaryTemplateMetaTypedDict
    direct_link: Nullable[TemplateCreateTemplateTemporaryDirectLinkTypedDict]
    user: TemplateCreateTemplateTemporaryUserTypedDict
    recipients: List[TemplateCreateTemplateTemporaryRecipientTypedDict]
    fields: List[TemplateCreateTemplateTemporaryFieldTypedDict]
    folder: Nullable[TemplateCreateTemplateTemporaryFolderTypedDict]
    envelope_items: List[TemplateCreateTemplateTemporaryEnvelopeItemTypedDict]
    template_document_data_id: NotRequired[str]


class TemplateCreateTemplateTemporaryTemplate(BaseModel):
    type: TemplateCreateTemplateTemporaryTypeResponse

    visibility: TemplateCreateTemplateTemporaryVisibilityResponse

    id: float

    external_id: Annotated[Nullable[str], pydantic.Field(alias="externalId")]

    title: str

    user_id: Annotated[float, pydantic.Field(alias="userId")]

    team_id: Annotated[float, pydantic.Field(alias="teamId")]

    auth_options: Annotated[
        Nullable[TemplateCreateTemplateTemporaryAuthOptions],
        pydantic.Field(alias="authOptions"),
    ]

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]

    updated_at: Annotated[str, pydantic.Field(alias="updatedAt")]

    public_title: Annotated[str, pydantic.Field(alias="publicTitle")]

    public_description: Annotated[str, pydantic.Field(alias="publicDescription")]

    folder_id: Annotated[Nullable[str], pydantic.Field(alias="folderId")]

    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]

    template_document_data: Annotated[
        TemplateCreateTemplateTemporaryTemplateDocumentData,
        pydantic.Field(alias="templateDocumentData"),
    ]

    template_meta: Annotated[
        TemplateCreateTemplateTemporaryTemplateMeta,
        pydantic.Field(alias="templateMeta"),
    ]

    direct_link: Annotated[
        Nullable[TemplateCreateTemplateTemporaryDirectLink],
        pydantic.Field(alias="directLink"),
    ]

    user: TemplateCreateTemplateTemporaryUser

    recipients: List[TemplateCreateTemplateTemporaryRecipient]

    fields: List[TemplateCreateTemplateTemporaryField]

    folder: Nullable[TemplateCreateTemplateTemporaryFolder]

    envelope_items: Annotated[
        List[TemplateCreateTemplateTemporaryEnvelopeItem],
        pydantic.Field(alias="envelopeItems"),
    ]

    template_document_data_id: Annotated[
        Optional[str], pydantic.Field(alias="templateDocumentDataId")
    ] = ""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["templateDocumentDataId"]
        nullable_fields = [
            "externalId",
            "authOptions",
            "folderId",
            "directLink",
            "folder",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class TemplateCreateTemplateTemporaryResponseTypedDict(TypedDict):
    r"""Successful response"""

    template: TemplateCreateTemplateTemporaryTemplateTypedDict
    upload_url: str


class TemplateCreateTemplateTemporaryResponse(BaseModel):
    r"""Successful response"""

    template: TemplateCreateTemplateTemporaryTemplate

    upload_url: Annotated[str, pydantic.Field(alias="uploadUrl")]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
from documenso_sdk.models import DocumensoError
from documenso_sdk.types import BaseModel
from documenso_sdk.utils import FieldMetadata, MultipartFormMetadata
from enum import Enum
import httpx
import io
import pydantic
from typing import IO, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


class EnvelopeUseRecipientTypedDict(TypedDict):
    id: float
    email: str
    name: NotRequired[str]
    signing_order: NotRequired[float]


class EnvelopeUseRecipient(BaseModel):
    id: float

    email: str

    name: Optional[str] = None

    signing_order: Annotated[Optional[float], pydantic.Field(alias="signingOrder")] = (
        None
    )


EnvelopeUseIdentifierTypedDict = TypeAliasType(
    "EnvelopeUseIdentifierTypedDict", Union[str, float]
)


EnvelopeUseIdentifier = TypeAliasType("EnvelopeUseIdentifier", Union[str, float])


class EnvelopeUseCustomDocumentDatumTypedDict(TypedDict):
    identifier: EnvelopeUseIdentifierTypedDict
    envelope_item_id: str


class EnvelopeUseCustomDocumentDatum(BaseModel):
    identifier: EnvelopeUseIdentifier

    envelope_item_id: Annotated[str, pydantic.Field(alias="envelopeItemId")]


class EnvelopeUseTypeDate(str, Enum):
    DATE = "date"


class EnvelopeUsePrefillFieldDateTypedDict(TypedDict):
    type: EnvelopeUseTypeDate
    id: float
    value: NotRequired[str]


class EnvelopeUsePrefillFieldDate(BaseModel):
    type: EnvelopeUseTypeDate

    id: float

    value: Optional[str] = None


class EnvelopeUseTypeDropdown(str, Enum):
    DROPDOWN = "dropdown"


class EnvelopeUsePrefillFieldDropdownTypedDict(TypedDict):
    type: EnvelopeUseTypeDropdown
    id: float
    label: NotRequired[str]
    value: NotRequired[str]


class EnvelopeUsePrefillFieldDropdown(BaseModel):
    type: EnvelopeUseTypeDropdown

    id: float

    label: Optional[str] = None

    value: Optional[str] = None


class EnvelopeUseTypeCheckbox(str, Enum):
    CHECKBOX = "checkbox"


class EnvelopeUsePrefillFieldCheckboxTypedDict(TypedDict):
    type: EnvelopeUseTypeCheckbox
    id: float
    label: NotRequired[str]
    value: NotRequired[List[str]]


class EnvelopeUsePrefillFieldCheckbox(BaseModel):
    type: EnvelopeUseTypeCheckbox

    id: float

    label: Optional[str] = None

    value: Optional[List[str]] = None


class EnvelopeUseTypeRadio(str, Enum):
    RADIO = "radio"


class EnvelopeUsePrefillFieldRadioTypedDict(TypedDict):
    type: EnvelopeUseTypeRadio
    id: float
    label: NotRequired[str]
    value: NotRequired[str]


class EnvelopeUsePrefillFieldRadio(BaseModel):
    type: EnvelopeUseTypeRadio

    id: float

    label: Optional[str] = None

    value: Optional[str] = None


class EnvelopeUseTypeNumber(str, Enum):
    NUMBER = "number"


class EnvelopeUsePrefillFieldNumberTypedDict(TypedDict):
    type: EnvelopeUseTypeNumber
    id: float
    label: NotRequired[str]
    placeholder: NotRequired[str]
    value: NotRequired[str]


class EnvelopeUsePrefillFieldNumber(BaseModel):
    type: EnvelopeUseTypeNumber

    id: float

    label: Optional[str] = None

    placeholder: Optional[str] = None

    value: Optional[str] = None


class EnvelopeUseTypeText(str, Enum):
    TEXT = "text"


class EnvelopeUsePrefillFieldTextTypedDict(TypedDict):
    type: EnvelopeUseTypeText
    id: float
    label: NotRequired[str]
    placeholder: NotRequired[str]
    value: NotRequired[str]


class EnvelopeUsePrefillFieldText(BaseModel):
    type: EnvelopeUseTypeText

    id: float

    label: Optional[str] = None

    placeholder: Optional[str] = None

    value: Optional[str] = None


EnvelopeUsePrefillFieldUnionTypedDict = TypeAliasType(
    "EnvelopeUsePrefillFieldUnionTypedDict",
    Union[
        EnvelopeUsePrefillFieldDateTypedDict,
        EnvelopeUsePrefillFieldRadioTypedDict,
        EnvelopeUsePrefillFieldCheckboxTypedDict,
        EnvelopeUsePrefillFieldDropdownTypedDict,
        EnvelopeUsePrefillFieldTextTypedDict,
        EnvelopeUsePrefillFieldNumberTypedDict,
    ],
)


EnvelopeUsePrefillFieldUnion = TypeAliasType(
    "EnvelopeUsePrefillFieldUnion",
    Union[
        EnvelopeUsePrefillFieldDate,
        EnvelopeUsePrefillFieldRadio,
        EnvelopeUsePrefillFieldCheckbox,
        EnvelopeUsePrefillFieldDropdown,
        EnvelopeUsePrefillFieldText,
        EnvelopeUsePrefillFieldNumber,
    ],
)


class EnvelopeUseDateFormat(str, Enum):
    YYYY_M_MDD_HH_MM_A = "yyyy-MM-dd hh:mm a"
    YYYY_M_MDD = "yyyy-MM-dd"
    DD_MM_SLASH_YYYY = "dd/MM/yyyy"
    MM_DD_SLASH_YYYY = "MM/dd/yyyy"
    YY_M_MDD = "yy-MM-dd"
    MMMM_DD_COMMA_YYYY = "MMMM dd, yyyy"
    EEEE_MMMM_DD_COMMA_YYYY = "EEEE, MMMM dd, yyyy"
    DD_MM_SLASH_YYYY_HH_MM_A = "dd/MM/yyyy hh:mm a"
    DD_MM_SLASH_YYYY_H_HMM = "dd/MM/yyyy HH:mm"
    MM_DD_SLASH_YYYY_HH_MM_A = "MM/dd/yyyy hh:mm a"
    MM_DD_SLASH_YYYY_H_HMM = "MM/dd/yyyy HH:mm"
    DD_DOT_MM_DOT_YYYY = "dd.MM.yyyy"
    DD_DOT_MM_DOT_YYYY_H_HMM = "dd.MM.yyyy HH:mm"
    YYYY_M_MDD_H_HMM = "yyyy-MM-dd HH:mm"
    YY_M_MDD_HH_MM_A = "yy-MM-dd hh:mm a"
    YY_M_MDD_H_HMM = "yy-MM-dd HH:mm"
    YYYY_M_MDD_H_HMMSS = "yyyy-MM-dd HH:mm:ss"
    MMMM_DD_COMMA_YYYY_HH_MM_A = "MMMM dd, yyyy hh:mm a"
    MMMM_DD_COMMA_YYYY_H_HMM = "MMMM dd, yyyy HH:mm"
    EEEE_MMMM_DD_COMMA_YYYY_HH_MM_A = "EEEE, MMMM dd, yyyy hh:mm a"
    EEEE_MMMM_DD_COMMA_YYYY_H_HMM = "EEEE, MMMM dd, yyyy HH:mm"
    ISO8601_FULL = "yyyy-MM-dd'T'HH:mm:ss.SSSXXX"


class EnvelopeUseDistributionMethod(str, Enum):
    EMAIL = "EMAIL"
    NONE = "NONE"


class EnvelopeUseEmailSettingsTypedDict(TypedDict):
    recipient_signing_request: NotRequired[bool]
    recipient_removed: NotRequired[bool]
    recipient_signed: NotRequired[bool]
    document_pending: NotRequired[bool]
    document_completed: NotRequired[bool]
    document_deleted: NotRequired[bool]
    owner_document_completed: NotRequired[bool]


class EnvelopeUseEmailSettings(BaseModel):
    recipient_signing_request: Annotated[
        Optional[bool], pydantic.Field(alias="recipientSigningRequest")
    ] = True

    recipient_removed: Annotated[
        Optional[bool], pydantic.Field(alias="recipientRemoved")
    ] = True

    recipient_signed: Annotated[
        Optional[bool], pydantic.Field(alias="recipientSigned")
    ] = True

    document_pending: Annotated[
        Optional[bool], pydantic.Field(alias="documentPending")
    ] = True

    document_completed: Annotated[
        Optional[bool], pydantic.Field(alias="documentCompleted")
    ] = True

    document_deleted: Annotated[
        Optional[bool], pydantic.Field(alias="documentDeleted")
    ] = True

    owner_document_completed: Annotated[
        Optional[bool], pydantic.Field(alias="ownerDocumentCompleted")
    ] = True


class EnvelopeUseLanguage(str, Enum):
    DE = "de"
    EN = "en"
    FR = "fr"
    ES = "es"
    IT = "it"
    PL = "pl"


class OverrideTypedDict(TypedDict):
    title: NotRequired[str]
    subject: NotRequired[str]
    message: NotRequired[str]
    timezone: NotRequired[str]
    date_format: NotRequired[EnvelopeUseDateFormat]
    redirect_url: NotRequired[str]
    distribution_method: NotRequired[EnvelopeUseDistributionMethod]
    email_settings: NotRequired[EnvelopeUseEmailSettingsTypedDict]
    language: NotRequired[EnvelopeUseLanguage]
    typed_signature_enabled: NotRequired[bool]
    upload_signature_enabled: NotRequired[bool]
    draw_signature_enabled: NotRequired[bool]
    allow_dictate_next_signer: NotRequired[bool]


class Override(BaseModel):
    title: Optional[str] = None

    subject: Optional[str] = None

    message: Optional[str] = None

    timezone: Optional[str] = None

    date_format: Annotated[
        Optional[EnvelopeUseDateFormat], pydantic.Field(alias="dateFormat")
    ] = None

    redirect_url: Annotated[Optional[str], pydantic.Field(alias="redirectUrl")] = None

    distribution_method: Annotated[
        Optional[EnvelopeUseDistributionMethod],
        pydantic.Field(alias="distributionMethod"),
    ] = None

    email_settings: Annotated[
        Optional[EnvelopeUseEmailSettings], pydantic.Field(alias="emailSettings")
    ] = None

    language: Optional[EnvelopeUseLanguage] = None

    typed_signature_enabled: Annotated[
        Optional[bool], pydantic.Field(alias="typedSignatureEnabled")
    ] = None

    upload_signature_enabled: Annotated[
        Optional[bool], pydantic.Field(alias="uploadSignatureEnabled")
    ] = None

    draw_signature_enabled: Annotated[
        Optional[bool], pydantic.Field(alias="drawSignatureEnabled")
    ] = None

    allow_dictate_next_signer: Annotated[
        Optional[bool], pydantic.Field(alias="allowDictateNextSigner")
    ] = None


class EnvelopeUseTypeLink(str, Enum):
    LINK = "link"


class EnvelopeUseAttachmentTypedDict(TypedDict):
    label: str
    data: str
    type: NotRequired[EnvelopeUseTypeLink]


class EnvelopeUseAttachment(BaseModel):
    label: str

    data: str

    type: Optional[EnvelopeUseTypeLink] = EnvelopeUseTypeLink.LINK


class EnvelopeUsePayloadTypedDict(TypedDict):
    envelope_id: str
    recipients: List[EnvelopeUseRecipientTypedDict]
    external_id: NotRequired[str]
    distribute_document: NotRequired[bool]
    custom_document_data: NotRequired[List[EnvelopeUseCustomDocumentDatumTypedDict]]
    folder_id: NotRequired[str]
    prefill_fields: NotRequired[List[EnvelopeUsePrefillFieldUnionTypedDict]]
    override: NotRequired[OverrideTypedDict]
    attachments: NotRequired[List[EnvelopeUseAttachmentTypedDict]]


class EnvelopeUsePayload(BaseModel):
    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]

    recipients: List[EnvelopeUseRecipient]

    external_id: Annotated[Optional[str], pydantic.Field(alias="externalId")] = None

    distribute_document: Annotated[
        Optional[bool], pydantic.Field(alias="distributeDocument")
    ] = None

    custom_document_data: Annotated[
        Optional[List[EnvelopeUseCustomDocumentDatum]],
        pydantic.Field(alias="customDocumentData"),
    ] = None

    folder_id: Annotated[Optional[str], pydantic.Field(alias="folderId")] = None

    prefill_fields: Annotated[
        Optional[List[EnvelopeUsePrefillFieldUnion]],
        pydantic.Field(alias="prefillFields"),
    ] = None

    override: Optional[Override] = None

    attachments: Optional[List[EnvelopeUseAttachment]] = None


class EnvelopeUseFileTypedDict(TypedDict):
    file_name: str
    content: Union[bytes, IO[bytes], io.BufferedReader]
    content_type: NotRequired[str]


class EnvelopeUseFile(BaseModel):
    file_name: Annotated[
        str, pydantic.Field(alias="fileName"), FieldMetadata(multipart=True)
    ]

    content: Annotated[
        Union[bytes, IO[bytes], io.BufferedReader],
        pydantic.Field(alias=""),
        FieldMetadata(multipart=MultipartFormMetadata(content=True)),
    ]

    content_type: Annotated[
        Optional[str],
        pydantic.Field(alias="Content-Type"),
        FieldMetadata(multipart=True),
    ] = None


class EnvelopeUseRequestTypedDict(TypedDict):
    payload: EnvelopeUsePayloadTypedDict
    files: NotRequired[List[EnvelopeUseFileTypedDict]]


class EnvelopeUseRequest(BaseModel):
    payload: Annotated[
        EnvelopeUsePayload, FieldMetadata(multipart=MultipartFormMetadata(json=True))
    ]

    files: Annotated[
        Optional[List[EnvelopeUseFile]],
        FieldMetadata(multipart=MultipartFormMetadata(file=True)),
    ] = None


class EnvelopeUseInternalServerErrorIssueTypedDict(TypedDict):
    message: str


class EnvelopeUseInternalServerErrorIssue(BaseModel):
    message: str


class EnvelopeUseInternalServerErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeUseInternalServerErrorIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeUseInternalServerError(DocumensoError):
    r"""Internal server error"""

    data: EnvelopeUseInternalServerErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeUseInternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeUseForbiddenIssueTypedDict(TypedDict):
    message: str


class EnvelopeUseForbiddenIssue(BaseModel):
    message: str


class EnvelopeUseForbiddenErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeUseForbiddenIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeUseForbiddenError(DocumensoError):
    r"""Insufficient access"""

    data: EnvelopeUseForbiddenErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeUseForbiddenErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeUseUnauthorizedIssueTypedDict(TypedDict):
    message: str


class EnvelopeUseUnauthorizedIssue(BaseModel):
    message: str


class EnvelopeUseUnauthorizedErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeUseUnauthorizedIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeUseUnauthorizedError(DocumensoError):
    r"""Authorization not provided"""

    data: EnvelopeUseUnauthorizedErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeUseUnauthorizedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeUseBadRequestIssueTypedDict(TypedDict):
    message: str


class EnvelopeUseBadRequestIssue(BaseModel):
    message: str


class EnvelopeUseBadRequestErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeUseBadRequestIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeUseBadRequestError(DocumensoError):
    r"""Invalid input data"""

    data: EnvelopeUseBadRequestErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeUseBadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeUseResponseTypedDict(TypedDict):
    r"""Successful response"""

    id: str


class EnvelopeUseResponse(BaseModel):
    r"""Successful response"""

    id: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
from documenso_sdk.models import DocumensoError
from documenso_sdk.types import BaseModel
import httpx
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, TypedDict


class EnvelopeDuplicateRequestTypedDict(TypedDict):
    envelope_id: str


class EnvelopeDuplicateRequest(BaseModel):
    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]


class EnvelopeDuplicateInternalServerErrorIssueTypedDict(TypedDict):
    message: str


class EnvelopeDuplicateInternalServerErrorIssue(BaseModel):
    message: str


class EnvelopeDuplicateInternalServerErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeDuplicateInternalServerErrorIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeDuplicateInternalServerError(DocumensoError):
    r"""Internal server error"""

    data: EnvelopeDuplicateInternalServerErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeDuplicateInternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeDuplicateForbiddenIssueTypedDict(TypedDict):
    message: str


class EnvelopeDuplicateForbiddenIssue(BaseModel):
    message: str


class EnvelopeDuplicateForbiddenErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeDuplicateForbiddenIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeDuplicateForbiddenError(DocumensoError):
    r"""Insufficient access"""

    data: EnvelopeDuplicateForbiddenErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeDuplicateForbiddenErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeDuplicateUnauthorizedIssueTypedDict(TypedDict):
    message: str


class EnvelopeDuplicateUnauthorizedIssue(BaseModel):
    message: str


class EnvelopeDuplicateUnauthorizedErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeDuplicateUnauthorizedIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeDuplicateUnauthorizedError(DocumensoError):
    r"""Authorization not provided"""

    data: EnvelopeDuplicateUnauthorizedErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeDuplicateUnauthorizedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeDuplicateBadRequestIssueTypedDict(TypedDict):
    message: str


class EnvelopeDuplicateBadRequestIssue(BaseModel):
    message: str


class EnvelopeDuplicateBadRequestErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeDuplicateBadRequestIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeDuplicateBadRequestError(DocumensoError):
    r"""Invalid input data"""

    data: EnvelopeDuplicateBadRequestErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeDuplicateBadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeDuplicateResponseTypedDict(TypedDict):
    r"""Successful response"""

    id: str


class EnvelopeDuplicateResponse(BaseModel):
    r"""Successful response"""

    id: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
from documenso_sdk.models import DocumensoError
from documenso_sdk.types import BaseModel
import httpx
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, TypedDict


class EnvelopeDeleteRequestTypedDict(TypedDict):
    envelope_id: str


class EnvelopeDeleteRequest(BaseModel):
    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]


class EnvelopeDeleteInternalServerErrorIssueTypedDict(TypedDict):
    message: str


class EnvelopeDeleteInternalServerErrorIssue(BaseModel):
    message: str


class EnvelopeDeleteInternalServerErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeDeleteInternalServerErrorIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeDeleteInternalServerError(DocumensoError):
    r"""Internal server error"""

    data: EnvelopeDeleteInternalServerErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeDeleteInternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeDeleteForbiddenIssueTypedDict(TypedDict):
    message: str


class EnvelopeDeleteForbiddenIssue(BaseModel):
    message: str


class EnvelopeDeleteForbiddenErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeDeleteForbiddenIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeDeleteForbiddenError(DocumensoError):
    r"""Insufficient access"""

    data: EnvelopeDeleteForbiddenErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeDeleteForbiddenErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeDeleteUnauthorizedIssueTypedDict(TypedDict):
    message: str


class EnvelopeDeleteUnauthorizedIssue(BaseModel):
    message: str


class EnvelopeDeleteUnauthorizedErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeDeleteUnauthorizedIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeDeleteUnauthorizedError(DocumensoError):
    r"""Authorization not provided"""

    data: EnvelopeDeleteUnauthorizedErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeDeleteUnauthorizedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeDeleteBadRequestIssueTypedDict(TypedDict):
    message: str


class EnvelopeDeleteBadRequestIssue(BaseModel):
    message: str


class EnvelopeDeleteBadRequestErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeDeleteBadRequestIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeDeleteBadRequestError(DocumensoError):
    r"""Invalid input data"""

    data: EnvelopeDeleteBadRequestErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeDeleteBadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeDeleteResponseTypedDict(TypedDict):
    r"""Successful response"""

    success: bool


class EnvelopeDeleteResponse(BaseModel):
    r"""Successful response"""

    success: bool



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
from documenso_sdk.models import DocumensoError
from documenso_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from enum import Enum
import httpx
import pydantic
from pydantic import model_serializer
from typing import Any, Dict, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


class TemplateCreateDocumentFromTemplateRecipientRequestTypedDict(TypedDict):
    id: float
    email: str
    name: NotRequired[str]


class TemplateCreateDocumentFromTemplateRecipientRequest(BaseModel):
    id: float

    email: str

    name: Optional[str] = None


class TemplateCreateDocumentFromTemplateCustomDocumentDatumTypedDict(TypedDict):
    document_data_id: str
    envelope_item_id: str


class TemplateCreateDocumentFromTemplateCustomDocumentDatum(BaseModel):
    document_data_id: Annotated[str, pydantic.Field(alias="documentDataId")]

    envelope_item_id: Annotated[str, pydantic.Field(alias="envelopeItemId")]


class TemplateCreateDocumentFromTemplatePrefillFieldTypeDate(str, Enum):
    DATE = "date"


class TemplateCreateDocumentFromTemplatePrefillFieldDateTypedDict(TypedDict):
    type: TemplateCreateDocumentFromTemplatePrefillFieldTypeDate
    id: float
    value: NotRequired[str]


class TemplateCreateDocumentFromTemplatePrefillFieldDate(BaseModel):
    type: TemplateCreateDocumentFromTemplatePrefillFieldTypeDate

    id: float

    value: Optional[str] = None


class TemplateCreateDocumentFromTemplatePrefillFieldTypeDropdown(str, Enum):
    DROPDOWN = "dropdown"


class TemplateCreateDocumentFromTemplatePrefillFieldDropdownTypedDict(TypedDict):
    type: TemplateCreateDocumentFromTemplatePrefillFieldTypeDropdown
    id: float
    label: NotRequired[str]
    value: NotRequired[str]


class TemplateCreateDocumentFromTemplatePrefillFieldDropdown(BaseModel):
    type: TemplateCreateDocumentFromTemplatePrefillFieldTypeDropdown

    id: float

    label: Optional[str] = None

    value: Optional[str] = None


class TemplateCreateDocumentFromTemplatePrefillFieldTypeCheckbox(str, Enum):
    CHECKBOX = "checkbox"


class TemplateCreateDocumentFromTemplatePrefillFieldCheckboxTypedDict(TypedDict):
    type: TemplateCreateDocumentFromTemplatePrefillFieldTypeCheckbox
    id: float
    label: NotRequired[str]
    value: NotRequired[List[str]]


class TemplateCreateDocumentFromTemplatePrefillFieldCheckbox(BaseModel):
    type: TemplateCreateDocumentFromTemplatePrefillFieldTypeCheckbox

    id: float

    label: Optional[str] = None

    value: Optional[List[str]] = None


class TemplateCreateDocumentFromTemplatePrefillFieldTypeRadio(str, Enum):
    RADIO = "radio"


class TemplateCreateDocumentFromTemplatePrefillFieldRadioTypedDict(TypedDict):
    type: TemplateCreateDocumentFromTemplatePrefillFieldTypeRadio
    id: float
    label: NotRequired[str]
    value: NotRequired[str]


class TemplateCreateDocumentFromTemplatePrefillFieldRadio(BaseModel):
    type: TemplateCreateDocumentFromTemplatePrefillFieldTypeRadio

    id: float

    label: Optional[str] = None

    value: Optional[str] = None


class TemplateCreateDocumentFromTemplatePrefillFieldTypeNumber(str, Enum):
    NUMBER = "number"


class TemplateCreateDocumentFromTemplatePrefillFieldNumberTypedDict(TypedDict):
    type: TemplateCreateDocumentFromTemplatePrefillFieldTypeNumber
    id: float
    label: NotRequired[str]
    placeholder: NotRequired[str]
    value: NotRequired[str]


class TemplateCreateDocumentFromTemplatePrefillFieldNumber(BaseModel):
    type: TemplateCreateDocumentFromTemplatePrefillFieldTypeNumber

    id: float

    label: Optional[str] = None

    placeholder: Optional[str] = None

    value: Optional[str] = None


class TemplateCreateDocumentFromTemplatePrefillFieldTypeText(str, Enum):
    TEXT = "text"


class TemplateCreateDocumentFromTemplatePrefillFieldTextTypedDict(TypedDict):
    type: TemplateCreateDocumentFromTemplatePrefillFieldTypeText
    id: float
    label: NotRequired[str]
    placeholder: NotRequired[str]
    value: NotRequired[str]


class TemplateCreateDocumentFromTemplatePrefillFieldText(BaseModel):
    type: TemplateCreateDocumentFromTemplatePrefillFieldTypeText

    id: float

    label: Optional[str] = None

    placeholder: Optional[str] = None

    value: Optional[str] = None


TemplateCreateDocumentFromTemplatePrefillFieldUnionTypedDict = TypeAliasType(
    "TemplateCreateDocumentFromTemplatePrefillFieldUnionTypedDict",
    Union[
        TemplateCreateDocumentFromTemplatePrefillFieldDateTypedDict,
        TemplateCreateDocumentFromTemplatePrefillFieldRadioTypedDict,
        TemplateCreateDocumentFromTemplatePrefillFieldCheckboxTypedDict,
        TemplateCreateDocumentFromTemplatePrefillFieldDropdownTypedDict,
        TemplateCreateDocumentFromTemplatePrefillFieldTextTypedDict,
        TemplateCreateDocumentFromTemplatePrefillFieldNumberTypedDict,
    ],
)


TemplateCreateDocumentFromTemplatePrefillFieldUnion = TypeAliasType(
    "TemplateCreateDocumentFromTemplatePrefillFieldUnion",
    Union[
        TemplateCreateDocumentFromTemplatePrefillFieldDate,
        TemplateCreateDocumentFromTemplatePrefillFieldRadio,
        TemplateCreateDocumentFromTemplatePrefillFieldCheckbox,
        TemplateCreateDocumentFromTemplatePrefillFieldDropdown,
        TemplateCreateDocumentFromTemplatePrefillFieldText,
        TemplateCreateDocumentFromTemplatePrefillFieldNumber,
    ],
)


class TemplateCreateDocumentFromTemplateRequestTypedDict(TypedDict):
    template_id: float
    recipients: List[TemplateCreateDocumentFromTemplateRecipientRequestTypedDict]
    distribute_document: NotRequired[bool]
    custom_document_data_id: NotRequired[str]
    custom_document_data: NotRequired[
        List[TemplateCreateDocumentFromTemplateCustomDocumentDatumTypedDict]
    ]
    folder_id: NotRequired[str]
    prefill_fields: NotRequired[
        List[TemplateCreateDocumentFromTemplatePrefillFieldUnionTypedDict]
    ]


class TemplateCreateDocumentFromTemplateRequest(BaseModel):
    template_id: Annotated[float, pydantic.Field(alias="templateId")]

    recipients: List[TemplateCreateDocumentFromTemplateRecipientRequest]

    distribute_document: Annotated[
        Optional[bool], pydantic.Field(alias="distributeDocument")
    ] = None

    custom_document_data_id: Annotated[
        Optional[str], pydantic.Field(alias="customDocumentDataId")
    ] = None

    custom_document_data: Annotated[
        Optional[List[TemplateCreateDocumentFromTemplateCustomDocumentDatum]],
        pydantic.Field(alias="customDocumentData"),
    ] = None

    folder_id: Annotated[Optional[str], pydantic.Field(alias="folderId")] = None

    prefill_fields: Annotated[
        Optional[List[TemplateCreateDocumentFromTemplatePrefillFieldUnion]],
        pydantic.Field(alias="prefillFields"),
    ] = None


class TemplateCreateDocumentFromTemplateInternalServerErrorIssueTypedDict(TypedDict):
    message: str


class TemplateCreateDocumentFromTemplateInternalServerErrorIssue(BaseModel):
    message: str


class TemplateCreateDocumentFromTemplateInternalServerErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[
        List[TemplateCreateDocumentFromTemplateInternalServerErrorIssue]
    ] = None


@dataclass(unsafe_hash=True)
class TemplateCreateDocumentFromTemplateInternalServerError(DocumensoError):
    r"""Internal server error"""

    data: TemplateCreateDocumentFromTemplateInternalServerErrorData = field(hash=False)

    def __init__(
        self,
        data: TemplateCreateDocumentFromTemplateInternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class TemplateCreateDocumentFromTemplateForbiddenIssueTypedDict(TypedDict):
    message: str


class TemplateCreateDocumentFromTemplateForbiddenIssue(BaseModel):
    message: str


class TemplateCreateDocumentFromTemplateForbiddenErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[TemplateCreateDocumentFromTemplateForbiddenIssue]] = None


@dataclass(unsafe_hash=True)
class TemplateCreateDocumentFromTemplateForbiddenError(DocumensoError):
    r"""Insufficient access"""

    data: TemplateCreateDocumentFromTemplateForbiddenErrorData = field(hash=False)

    def __init__(
        self,
        data: TemplateCreateDocumentFromTemplateForbiddenErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class TemplateCreateDocumentFromTemplateUnauthorizedIssueTypedDict(TypedDict):
    message: str


class TemplateCreateDocumentFromTemplateUnauthorizedIssue(BaseModel):
    message: str


class TemplateCreateDocumentFromTemplateUnauthorizedErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[TemplateCreateDocumentFromTemplateUnauthorizedIssue]] = None


@dataclass(unsafe_hash=True)
class TemplateCreateDocumentFromTemplateUnauthorizedError(DocumensoError):
    r"""Authorization not provided"""

    data: TemplateCreateDocumentFromTemplateUnauthorizedErrorData = field(hash=False)

    def __init__(
        self,
        data: TemplateCreateDocumentFromTemplateUnauthorizedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class TemplateCreateDocumentFromTemplateBadRequestIssueTypedDict(TypedDict):
    message: str


class TemplateCreateDocumentFromTemplateBadRequestIssue(BaseModel):
    message: str


class TemplateCreateDocumentFromTemplateBadRequestErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[TemplateCreateDocumentFromTemplateBadRequestIssue]] = None


@dataclass(unsafe_hash=True)
class TemplateCreateDocumentFromTemplateBadRequestError(DocumensoError):
    r"""Invalid input data"""

    data: TemplateCreateDocumentFromTemplateBadRequestErrorData = field(hash=False)

    def __init__(
        self,
        data: TemplateCreateDocumentFromTemplateBadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class TemplateCreateDocumentFromTemplateVisibility(str, Enum):
    EVERYONE = "EVERYONE"
    MANAGER_AND_ABOVE = "MANAGER_AND_ABOVE"
    ADMIN = "ADMIN"


class TemplateCreateDocumentFromTemplateStatus(str, Enum):
    DRAFT = "DRAFT"
    PENDING = "PENDING"
    COMPLETED = "COMPLETED"
    REJECTED = "REJECTED"


class TemplateCreateDocumentFromTemplateSource(str, Enum):
    DOCUMENT = "DOCUMENT"
    TEMPLATE = "TEMPLATE"
    TEMPLATE_DIRECT_LINK = "TEMPLATE_DIRECT_LINK"


class TemplateCreateDocumentFromTemplateGlobalAccessAuth(str, Enum):
    ACCOUNT = "ACCOUNT"
    TWO_FACTOR_AUTH = "TWO_FACTOR_AUTH"


class TemplateCreateDocumentFromTemplateGlobalActionAuth(str, Enum):
    ACCOUNT = "ACCOUNT"
    PASSKEY = "PASSKEY"
    TWO_FACTOR_AUTH = "TWO_FACTOR_AUTH"
    PASSWORD = "PASSWORD"


class TemplateCreateDocumentFromTemplateAuthOptionsTypedDict(TypedDict):
    global_access_auth: List[TemplateCreateDocumentFromTemplateGlobalAccessAuth]
    global_action_auth: List[TemplateCreateDocumentFromTemplateGlobalActionAuth]


class TemplateCreateDocumentFromTemplateAuthOptions(BaseModel):
    global_access_auth: Annotated[
        List[TemplateCreateDocumentFromTemplateGlobalAccessAuth],
        pydantic.Field(alias="globalAccessAuth"),
    ]

    global_action_auth: Annotated[
        List[TemplateCreateDocumentFromTemplateGlobalActionAuth],
        pydantic.Field(alias="globalActionAuth"),
    ]


TemplateCreateDocumentFromTemplateFormValuesTypedDict = TypeAliasType(
    "TemplateCreateDocumentFromTemplateFormValuesTypedDict", Union[str, bool, float]
)


TemplateCreateDocumentFromTemplateFormValues = TypeAliasType(
    "TemplateCreateDocumentFromTemplateFormValues", Union[str, bool, float]
)


class TemplateCreateDocumentFromTemplateDocumentDataType(str, Enum):
    S3_PATH = "S3_PATH"
    BYTES = "BYTES"
    BYTES_64 = "BYTES_64"


class TemplateCreateDocumentFromTemplateDocumentDataTypedDict(TypedDict):
    type: TemplateCreateDocumentFromTemplateDocumentDataType
    id: str
    data: str
    initial_data: str
    envelope_item_id: str


class TemplateCreateDocumentFromTemplateDocumentData(BaseModel):
    type: TemplateCreateDocumentFromTemplateDocumentDataType

    id: str

    data: str

    initial_data: Annotated[str, pydantic.Field(alias="initialData")]

    envelope_item_id: Annotated[str, pydantic.Field(alias="envelopeItemId")]


class TemplateCreateDocumentFromTemplateSigningOrder(str, Enum):
    PARALLEL = "PARALLEL"
    SEQUENTIAL = "SEQUENTIAL"


class TemplateCreateDocumentFromTemplateDistributionMethod(str, Enum):
    EMAIL = "EMAIL"
    NONE = "NONE"


class TemplateCreateDocumentFromTemplateEmailSettingsTypedDict(TypedDict):
    recipient_signing_request: NotRequired[bool]
    recipient_removed: NotRequired[bool]
    recipient_signed: NotRequired[bool]
    document_pending: NotRequired[bool]
    document_completed: NotRequired[bool]
    document_deleted: NotRequired[bool]
    owner_document_completed: NotRequired[bool]


class TemplateCreateDocumentFromTemplateEmailSettings(BaseModel):
    recipient_signing_request: Annotated[
        Optional[bool], pydantic.Field(alias="recipientSigningRequest")
    ] = True

    recipient_removed: Annotated[
        Optional[bool], pydantic.Field(alias="recipientRemoved")
    ] = True

    recipient_signed: Annotated[
        Optional[bool], pydantic.Field(alias="recipientSigned")
    ] = True

    document_pending: Annotated[
        Optional[bool], pydantic.Field(alias="documentPending")
    ] = True

    document_completed: Annotated[
        Optional[bool], pydantic.Field(alias="documentCompleted")
    ] = True

    document_deleted: Annotated[
        Optional[bool], pydantic.Field(alias="documentDeleted")
    ] = True

    owner_document_completed: Annotated[
        Optional[bool], pydantic.Field(alias="ownerDocumentCompleted")
    ] = True


class TemplateCreateDocumentFromTemplateDocumentMetaTypedDict(TypedDict):
    signing_order: TemplateCreateDocumentFromTemplateSigningOrder
    distribution_method: TemplateCreateDocumentFromTemplateDistributionMethod
    id: str
    subject: Nullable[str]
    message: Nullable[str]
    timezone: Nullable[str]
    date_format: Nullable[str]
    redirect_url: Nullable[str]
    typed_signature_enabled: bool
    upload_signature_enabled: bool
    draw_signature_enabled: bool
    allow_dictate_next_signer: bool
    language: str
    email_settings: Nullable[TemplateCreateDocumentFromTemplateEmailSettingsTypedDict]
    email_id: Nullable[str]
    email_reply_to: Nullable[str]
    password: NotRequired[Nullable[str]]
    document_id: NotRequired[float]


class TemplateCreateDocumentFromTemplateDocumentMeta(BaseModel):
    signing_order: Annotated[
        TemplateCreateDocumentFromTemplateSigningOrder,
        pydantic.Field(alias="signingOrder"),
    ]

    distribution_method: Annotated[
        TemplateCreateDocumentFromTemplateDistributionMethod,
        pydantic.Field(alias="distributionMethod"),
    ]

    id: str

    subject: Nullable[str]

    message: Nullable[str]

    timezone: Nullable[str]

    date_format: Annotated[Nullable[str], pydantic.Field(alias="dateFormat")]

    redirect_url: Annotated[Nullable[str], pydantic.Field(alias="redirectUrl")]

    typed_signature_enabled: Annotated[
        bool, pydantic.Field(alias="typedSignatureEnabled")
    ]

    upload_signature_enabled: Annotated[
        bool, pydantic.Field(alias="uploadSignatureEnabled")
    ]

    draw_signature_enabled: Annotated[
        bool, pydantic.Field(alias="drawSignatureEnabled")
    ]

    allow_dictate_next_signer: Annotated[
        bool, pydantic.Field(alias="allowDictateNextSigner")
    ]

    language: str

    email_settings: Annotated[
        Nullable[TemplateCreateDocumentFromTemplateEmailSettings],
        pydantic.Field(alias="emailSettings"),
    ]

    email_id: Annotated[Nullable[str], pydantic.Field(alias="emailId")]

    email_reply_to: Annotated[Nullable[str], pydantic.Field(alias="emailReplyTo")]

    password: OptionalNullable[str] = None

    document_id: Annotated[Optional[float], pydantic.Field(alias="documentId")] = -1

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["password", "documentId"]
        nullable_fields = [
            "subject",
            "message",
            "timezone",
            "dateFormat",
            "redirectUrl",
            "emailSettings",
            "emailId",
            "emailReplyTo",
            "password",
        ]
        null_default_fields = ["password"]

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class TemplateCreateDocumentFromTemplateEnvelopeItemTypedDict(TypedDict):
    id: str
    envelope_id: str


class TemplateCreateDocumentFromTemplateEnvelopeItem(BaseModel):
    id: str

    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]


class TemplateCreateDocumentFromTemplateFolderType(str, Enum):
    DOCUMENT = "DOCUMENT"
    TEMPLATE = "TEMPLATE"


class TemplateCreateDocumentFromTemplateFolderVisibility(str, Enum):
    EVERYONE = "EVERYONE"
    MANAGER_AND_ABOVE = "MANAGER_AND_ABOVE"
    ADMIN = "ADMIN"


class TemplateCreateDocumentFromTemplateFolderTypedDict(TypedDict):
    id: str
    name: str
    type: TemplateCreateDocumentFromTemplateFolderType
    visibility: TemplateCreateDocumentFromTemplateFolderVisibility
    user_id: float
    team_id: float
    pinned: bool
    parent_id: Nullable[str]
    created_at: str
    updated_at: str


class TemplateCreateDocumentFromTemplateFolder(BaseModel):
    id: str

    name: str

    type: TemplateCreateDocumentFromTemplateFolderType

    visibility: TemplateCreateDocumentFromTemplateFolderVisibility

    user_id: Annotated[float, pydantic.Field(alias="userId")]

    team_id: Annotated[float, pydantic.Field(alias="teamId")]

    pinned: bool

    parent_id: Annotated[Nullable[str], pydantic.Field(alias="parentId")]

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]

    updated_at: Annotated[str, pydantic.Field(alias="updatedAt")]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["parentId"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class TemplateCreateDocumentFromTemplateRole(str, Enum):
    CC = "CC"
    SIGNER = "SIGNER"
    VIEWER = "VIEWER"
    APPROVER = "APPROVER"
    ASSISTANT = "ASSISTANT"


class TemplateCreateDocumentFromTemplateReadStatus(str, Enum):
    NOT_OPENED = "NOT_OPENED"
    OPENED = "OPENED"


class TemplateCreateDocumentFromTemplateSigningStatus(str, Enum):
    NOT_SIGNED = "NOT_SIGNED"
    SIGNED = "SIGNED"
    REJECTED = "REJECTED"


class TemplateCreateDocumentFromTemplateSendStatus(str, Enum):
    NOT_SENT = "NOT_SENT"
    SENT = "SENT"


class TemplateCreateDocumentFromTemplateAccessAuth(str, Enum):
    ACCOUNT = "ACCOUNT"
    TWO_FACTOR_AUTH = "TWO_FACTOR_AUTH"


class TemplateCreateDocumentFromTemplateActionAuth(str, Enum):
    ACCOUNT = "ACCOUNT"
    PASSKEY = "PASSKEY"
    TWO_FACTOR_AUTH = "TWO_FACTOR_AUTH"
    PASSWORD = "PASSWORD"
    EXPLICIT_NONE = "EXPLICIT_NONE"


class TemplateCreateDocumentFromTemplateRecipientAuthOptionsTypedDict(TypedDict):
    access_auth: List[TemplateCreateDocumentFromTemplateAccessAuth]
    action_auth: List[TemplateCreateDocumentFromTemplateActionAuth]


class TemplateCreateDocumentFromTemplateRecipientAuthOptions(BaseModel):
    access_auth: Annotated[
        List[TemplateCreateDocumentFromTemplateAccessAuth],
        pydantic.Field(alias="accessAuth"),
    ]

    action_auth: Annotated[
        List[TemplateCreateDocumentFromTemplateActionAuth],
        pydantic.Field(alias="actionAuth"),
    ]


class TemplateCreateDocumentFromTemplateRecipientResponseTypedDict(TypedDict):
    envelope_id: str
    role: TemplateCreateDocumentFromTemplateRole
    read_status: TemplateCreateDocumentFromTemplateReadStatus
    signing_status: TemplateCreateDocumentFromTemplateSigningStatus
    send_status: TemplateCreateDocumentFromTemplateSendStatus
    id: float
    email: str
    name: str
    token: str
    document_deleted_at: Nullable[str]
    expired: Nullable[str]
    signed_at: Nullable[str]
    auth_options: Nullable[
        TemplateCreateDocumentFromTemplateRecipientAuthOptionsTypedDict
    ]
    signing_order: Nullable[float]
    rejection_reason: Nullable[str]
    document_id: NotRequired[Nullable[float]]
    template_id: NotRequired[Nullable[float]]


class TemplateCreateDocumentFromTemplateRecipientResponse(BaseModel):
    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]

    role: TemplateCreateDocumentFromTemplateRole

    read_status: Annotated[
        TemplateCreateDocumentFromTemplateReadStatus, pydantic.Field(alias="readStatus")
    ]

    signing_status: Annotated[
        TemplateCreateDocumentFromTemplateSigningStatus,
        pydantic.Field(alias="signingStatus"),
    ]

    send_status: Annotated[
        TemplateCreateDocumentFromTemplateSendStatus, pydantic.Field(alias="sendStatus")
    ]

    id: float

    email: str

    name: str

    token: str

    document_deleted_at: Annotated[
        Nullable[str], pydantic.Field(alias="documentDeletedAt")
    ]

    expired: Nullable[str]

    signed_at: Annotated[Nullable[str], pydantic.Field(alias="signedAt")]

    auth_options: Annotated[
        Nullable[TemplateCreateDocumentFromTemplateRecipientAuthOptions],
        pydantic.Field(alias="authOptions"),
    ]

    signing_order: Annotated[Nullable[float], pydantic.Field(alias="signingOrder")]

    rejection_reason: Annotated[Nullable[str], pydantic.Field(alias="rejectionReason")]

    document_id: Annotated[
        OptionalNullable[float], pydantic.Field(alias="documentId")
    ] = UNSET

    template_id: Annotated[
        OptionalNullable[float], pydantic.Field(alias="templateId")
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["documentId", "templateId"]
        nullable_fields = [
            "documentDeletedAt",
            "expired",
            "signedAt",
            "authOptions",
            "signingOrder",
            "rejectionReason",
            "documentId",
            "templateId",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class TemplateCreateDocumentFromTemplateFieldType(str, Enum):
    SIGNATURE = "SIGNATURE"
    FREE_SIGNATURE = "FREE_SIGNATURE"
    INITIALS = "INITIALS"
    NAME = "NAME"
    EMAIL = "EMAIL"
    DATE = "DATE"
    TEXT = "TEXT"
    NUMBER = "NUMBER"
    RADIO = "RADIO"
    CHECKBOX = "CHECKBOX"
    DROPDOWN = "DROPDOWN"


class TemplateCreateDocumentFromTemplateFieldMetaTypeDropdown(str, Enum):
    DROPDOWN = "dropdown"


class TemplateCreateDocumentFromTemplateValue3TypedDict(TypedDict):
    value: str


class TemplateCreateDocumentFromTemplateValue3(BaseModel):
    value: str


class TemplateCreateDocumentFromTemplateFieldMetaDropdownTypedDict(TypedDict):
    type: TemplateCreateDocumentFromTemplateFieldMetaTypeDropdown
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[TemplateCreateDocumentFromTemplateValue3TypedDict]]
    default_value: NotRequired[str]


class TemplateCreateDocumentFromTemplateFieldMetaDropdown(BaseModel):
    type: TemplateCreateDocumentFromTemplateFieldMetaTypeDropdown

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[TemplateCreateDocumentFromTemplateValue3]] = None

    default_value: Annotated[Optional[str], pydantic.Field(alias="defaultValue")] = None


class TemplateCreateDocumentFromTemplateFieldMetaTypeCheckbox(str, Enum):
    CHECKBOX = "checkbox"


class TemplateCreateDocumentFromTemplateValue2TypedDict(TypedDict):
    id: float
    checked: bool
    value: str


class TemplateCreateDocumentFromTemplateValue2(BaseModel):
    id: float

    checked: bool

    value: str


class TemplateCreateDocumentFromTemplateDirection2(str, Enum):
    VERTICAL = "vertical"
    HORIZONTAL = "horizontal"


class TemplateCreateDocumentFromTemplateFieldMetaCheckboxTypedDict(TypedDict):
    type: TemplateCreateDocumentFromTemplateFieldMetaTypeCheckbox
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[TemplateCreateDocumentFromTemplateValue2TypedDict]]
    validation_rule: NotRequired[str]
    validation_length: NotRequired[float]
    direction: NotRequired[TemplateCreateDocumentFromTemplateDirection2]


class TemplateCreateDocumentFromTemplateFieldMetaCheckbox(BaseModel):
    type: TemplateCreateDocumentFromTemplateFieldMetaTypeCheckbox

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[TemplateCreateDocumentFromTemplateValue2]] = None

    validation_rule: Annotated[
        Optional[str], pydantic.Field(alias="validationRule")
    ] = None

    validation_length: Annotated[
        Optional[float], pydantic.Field(alias="validationLength")
    ] = None

    direction: Optional[TemplateCreateDocumentFromTemplateDirection2] = (
        TemplateCreateDocumentFromTemplateDirection2.VERTICAL
    )


class TemplateCreateDocumentFromTemplateFieldMetaTypeRadio(str, Enum):
    RADIO = "radio"


class TemplateCreateDocumentFromTemplateValue1TypedDict(TypedDict):
    id: float
    checked: bool
    value: str


class TemplateCreateDocumentFromTemplateValue1(BaseModel):
    id: float

    checked: bool

    value: str


class TemplateCreateDocumentFromTemplateDirection1(str, Enum):
    VERTICAL = "vertical"
    HORIZONTAL = "horizontal"


class TemplateCreateDocumentFromTemplateFieldMetaRadioTypedDict(TypedDict):
    type: TemplateCreateDocumentFromTemplateFieldMetaTypeRadio
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[TemplateCreateDocumentFromTemplateValue1TypedDict]]
    direction: NotRequired[TemplateCreateDocumentFromTemplateDirection1]


class TemplateCreateDocumentFromTemplateFieldMetaRadio(BaseModel):
    type: TemplateCreateDocumentFromTemplateFieldMetaTypeRadio

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[TemplateCreateDocumentFromTemplateValue1]] = None

    direction: Optional[TemplateCreateDocumentFromTemplateDirection1] = (
        TemplateCreateDocumentFromTemplateDirection1.VERTICAL
    )


class TemplateCreateDocumentFromTemplateFieldMetaTypeNumber(str, Enum):
    NUMBER = "number"


class TemplateCreateDocumentFromTemplateTextAlign6(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class TemplateCreateDocumentFromTemplateVerticalAlign2(str, Enum):
    TOP = "top"
    MIDDLE = "middle"
    BOTTOM = "bottom"


class TemplateCreateDocumentFromTemplateFieldMetaNumberTypedDict(TypedDict):
    type: TemplateCreateDocumentFromTemplateFieldMetaTypeNumber
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    number_format: NotRequired[Nullable[str]]
    value: NotRequired[str]
    min_value: NotRequired[Nullable[float]]
    max_value: NotRequired[Nullable[float]]
    text_align: NotRequired[TemplateCreateDocumentFromTemplateTextAlign6]
    line_height: NotRequired[Nullable[float]]
    letter_spacing: NotRequired[Nullable[float]]
    vertical_align: NotRequired[
        Nullable[TemplateCreateDocumentFromTemplateVerticalAlign2]
    ]


class TemplateCreateDocumentFromTemplateFieldMetaNumber(BaseModel):
    type: TemplateCreateDocumentFromTemplateFieldMetaTypeNumber

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    number_format: Annotated[
        OptionalNullable[str], pydantic.Field(alias="numberFormat")
    ] = UNSET

    value: Optional[str] = None

    min_value: Annotated[OptionalNullable[float], pydantic.Field(alias="minValue")] = (
        UNSET
    )

    max_value: Annotated[OptionalNullable[float], pydantic.Field(alias="maxValue")] = (
        UNSET
    )

    text_align: Annotated[
        Optional[TemplateCreateDocumentFromTemplateTextAlign6],
        pydantic.Field(alias="textAlign"),
    ] = None

    line_height: Annotated[
        OptionalNullable[float], pydantic.Field(alias="lineHeight")
    ] = UNSET

    letter_spacing: Annotated[
        OptionalNullable[float], pydantic.Field(alias="letterSpacing")
    ] = UNSET

    vertical_align: Annotated[
        OptionalNullable[TemplateCreateDocumentFromTemplateVerticalAlign2],
        pydantic.Field(alias="verticalAlign"),
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "label",
            "placeholder",
            "required",
            "readOnly",
            "fontSize",
            "numberFormat",
            "value",
            "minValue",
            "maxValue",
            "textAlign",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        nullable_fields = [
            "numberFormat",
            "minValue",
            "maxValue",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class TemplateCreateDocumentFromTemplateFieldMetaTypeText(str, Enum):
    TEXT = "text"


class TemplateCreateDocumentFromTemplateTextAlign5(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class TemplateCreateDocumentFromTemplateVerticalAlign1(str, Enum):
    TOP = "top"
    MIDDLE = "middle"
    BOTTOM = "bottom"


class TemplateCreateDocumentFromTemplateFieldMetaTextTypedDict(TypedDict):
    type: TemplateCreateDocumentFromTemplateFieldMetaTypeText
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text: NotRequired[str]
    character_limit: NotRequired[float]
    text_align: NotRequired[TemplateCreateDocumentFromTemplateTextAlign5]
    line_height: NotRequired[Nullable[float]]
    letter_spacing: NotRequired[Nullable[float]]
    vertical_align: NotRequired[
        Nullable[TemplateCreateDocumentFromTemplateVerticalAlign1]
    ]


class TemplateCreateDocumentFromTemplateFieldMetaText(BaseModel):
    type: TemplateCreateDocumentFromTemplateFieldMetaTypeText

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text: Optional[str] = None

    character_limit: Annotated[
        Optional[float], pydantic.Field(alias="characterLimit")
    ] = None

    text_align: Annotated[
        Optional[TemplateCreateDocumentFromTemplateTextAlign5],
        pydantic.Field(alias="textAlign"),
    ] = None

    line_height: Annotated[
        OptionalNullable[float], pydantic.Field(alias="lineHeight")
    ] = UNSET

    letter_spacing: Annotated[
        OptionalNullable[float], pydantic.Field(alias="letterSpacing")
    ] = UNSET

    vertical_align: Annotated[
        OptionalNullable[TemplateCreateDocumentFromTemplateVerticalAlign1],
        pydantic.Field(alias="verticalAlign"),
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "label",
            "placeholder",
            "required",
            "readOnly",
            "fontSize",
            "text",
            "characterLimit",
            "textAlign",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        nullable_fields = ["lineHeight", "letterSpacing", "verticalAlign"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class TemplateCreateDocumentFromTemplateFieldMetaTypeDate(str, Enum):
    DATE = "date"


class TemplateCreateDocumentFromTemplateTextAlign4(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class TemplateCreateDocumentFromTemplateFieldMetaDateTypedDict(TypedDict):
    type: TemplateCreateDocumentFromTemplateFieldMetaTypeDate
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[TemplateCreateDocumentFromTemplateTextAlign4]


class TemplateCreateDocumentFromTemplateFieldMetaDate(BaseModel):
    type: TemplateCreateDocumentFromTemplateFieldMetaTypeDate

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[TemplateCreateDocumentFromTemplateTextAlign4],
        pydantic.Field(alias="textAlign"),
    ] = None


class TemplateCreateDocumentFromTemplateTypeEmail(str, Enum):
    EMAIL = "email"


class TemplateCreateDocumentFromTemplateTextAlign3(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class TemplateCreateDocumentFromTemplateFieldMetaEmailTypedDict(TypedDict):
    type: TemplateCreateDocumentFromTemplateTypeEmail
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[TemplateCreateDocumentFromTemplateTextAlign3]


class TemplateCreateDocumentFromTemplateFieldMetaEmail(BaseModel):
    type: TemplateCreateDocumentFromTemplateTypeEmail

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[TemplateCreateDocumentFromTemplateTextAlign3],
        pydantic.Field(alias="textAlign"),
    ] = None


class TemplateCreateDocumentFromTemplateTypeName(str, Enum):
    NAME = "name"


class TemplateCreateDocumentFromTemplateTextAlign2(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class TemplateCreateDocumentFromTemplateFieldMetaNameTypedDict(TypedDict):
    type: TemplateCreateDocumentFromTemplateTypeName
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[TemplateCreateDocumentFromTemplateTextAlign2]


class TemplateCreateDocumentFromTemplateFieldMetaName(BaseModel):
    type: TemplateCreateDocumentFromTemplateTypeName

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[TemplateCreateDocumentFromTemplateTextAlign2],
        pydantic.Field(alias="textAlign"),
    ] = None


class TemplateCreateDocumentFromTemplateTypeInitials(str, Enum):
    INITIALS = "initials"


class TemplateCreateDocumentFromTemplateTextAlign1(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class TemplateCreateDocumentFromTemplateFieldMetaInitialsTypedDict(TypedDict):
    type: TemplateCreateDocumentFromTemplateTypeInitials
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[TemplateCreateDocumentFromTemplateTextAlign1]


class TemplateCreateDocumentFromTemplateFieldMetaInitials(BaseModel):
    type: TemplateCreateDocumentFromTemplateTypeInitials

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[TemplateCreateDocumentFromTemplateTextAlign1],
        pydantic.Field(alias="textAlign"),
    ] = None


class TemplateCreateDocumentFromTemplateTypeSignature(str, Enum):
    SIGNATURE = "signature"


class TemplateCreateDocumentFromTemplateFieldMetaSignatureTypedDict(TypedDict):
    type: TemplateCreateDocumentFromTemplateTypeSignature
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]


class TemplateCreateDocumentFromTemplateFieldMetaSignature(BaseModel):
    type: TemplateCreateDocumentFromTemplateTypeSignature

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12


TemplateCreateDocumentFromTemplateFieldMetaUnionTypedDict = TypeAliasType(
    "TemplateCreateDocumentFromTemplateFieldMetaUnionTypedDict",
    Union[
        TemplateCreateDocumentFromTemplateFieldMetaSignatureTypedDict,
        TemplateCreateDocumentFromTemplateFieldMetaInitialsTypedDict,
        TemplateCreateDocumentFromTemplateFieldMetaNameTypedDict,
        TemplateCreateDocumentFromTemplateFieldMetaEmailTypedDict,
        TemplateCreateDocumentFromTemplateFieldMetaDateTypedDict,
        TemplateCreateDocumentFromTemplateFieldMetaRadioTypedDict,
        TemplateCreateDocumentFromTemplateFieldMetaDropdownTypedDict,
        TemplateCreateDocumentFromTemplateFieldMetaCheckboxTypedDict,
        TemplateCreateDocumentFromTemplateFieldMetaTextTypedDict,
        TemplateCreateDocumentFromTemplateFieldMetaNumberTypedDict,
    ],
)


TemplateCreateDocumentFromTemplateFieldMetaUnion = TypeAliasType(
    "TemplateCreateDocumentFromTemplateFieldMetaUnion",
    Union[
        TemplateCreateDocumentFromTemplateFieldMetaSignature,
        TemplateCreateDocumentFromTemplateFieldMetaInitials,
        TemplateCreateDocumentFromTemplateFieldMetaName,
        TemplateCreateDocumentFromTemplateFieldMetaEmail,
        TemplateCreateDocumentFromTemplateFieldMetaDate,
        TemplateCreateDocumentFromTemplateFieldMetaRadio,
        TemplateCreateDocumentFromTemplateFieldMetaDropdown,
        TemplateCreateDocumentFromTemplateFieldMetaCheckbox,
        TemplateCreateDocumentFromTemplateFieldMetaText,
        TemplateCreateDocumentFromTemplateFieldMetaNumber,
    ],
)


class TemplateCreateDocumentFromTemplateFieldTypedDict(TypedDict):
    envelope_id: str
    envelope_item_id: str
    type: TemplateCreateDocumentFromTemplateFieldType
    id: float
    secondary_id: str
    recipient_id: float
    page: float
    position_x: Any
    position_y: Any
    width: Any
    height: Any
    custom_text: str
    inserted: bool
    field_meta: Nullable[TemplateCreateDocumentFromTemplateFieldMetaUnionTypedDict]
    document_id: NotRequired[Nullable[float]]
    template_id: NotRequired[Nullable[float]]


class TemplateCreateDocumentFromTemplateField(BaseModel):
    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]

    envelope_item_id: Annotated[str, pydantic.Field(alias="envelopeItemId")]

    type: TemplateCreateDocumentFromTemplateFieldType

    id: float

    secondary_id: Annotated[str, pydantic.Field(alias="secondaryId")]

    recipient_id: Annotated[float, pydantic.Field(alias="recipientId")]

    page: float

    position_x: Annotated[Any, pydantic.Field(alias="positionX")]

    position_y: Annotated[Any, pydantic.Field(alias="positionY")]

    width: Any

    height: Any

    custom_text: Annotated[str, pydantic.Field(alias="customText")]

    inserted: bool

    field_meta: Annotated[
        Nullable[TemplateCreateDocumentFromTemplateFieldMetaUnion],
        pydantic.Field(alias="fieldMeta"),
    ]

    document_id: Annotated[
        OptionalNullable[float], pydantic.Field(alias="documentId")
    ] = UNSET

    template_id: Annotated[
        OptionalNullable[float], pydantic.Field(alias="templateId")
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["documentId", "templateId"]
        nullable_fields = ["fieldMeta", "documentId", "templateId"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class TemplateCreateDocumentFromTemplateResponseTypedDict(TypedDict):
    r"""Successful response"""

    visibility: TemplateCreateDocumentFromTemplateVisibility
    status: TemplateCreateDocumentFromTemplateStatus
    source: TemplateCreateDocumentFromTemplateSource
    id: float
    external_id: Nullable[str]
    user_id: float
    auth_options: Nullable[TemplateCreateDocumentFromTemplateAuthOptionsTypedDict]
    form_values: Nullable[
        Dict[str, TemplateCreateDocumentFromTemplateFormValuesTypedDict]
    ]
    title: str
    created_at: str
    updated_at: str
    completed_at: Nullable[str]
    deleted_at: Nullable[str]
    team_id: float
    folder_id: Nullable[str]
    envelope_id: str
    internal_version: float
    document_data: TemplateCreateDocumentFromTemplateDocumentDataTypedDict
    document_meta: TemplateCreateDocumentFromTemplateDocumentMetaTypedDict
    envelope_items: List[TemplateCreateDocumentFromTemplateEnvelopeItemTypedDict]
    folder: Nullable[TemplateCreateDocumentFromTemplateFolderTypedDict]
    recipients: List[TemplateCreateDocumentFromTemplateRecipientResponseTypedDict]
    fields: List[TemplateCreateDocumentFromTemplateFieldTypedDict]
    template_id: NotRequired[Nullable[float]]
    document_data_id: NotRequired[str]


class TemplateCreateDocumentFromTemplateResponse(BaseModel):
    r"""Successful response"""

    visibility: TemplateCreateDocumentFromTemplateVisibility

    status: TemplateCreateDocumentFromTemplateStatus

    source: TemplateCreateDocumentFromTemplateSource

    id: float

    external_id: Annotated[Nullable[str], pydantic.Field(alias="externalId")]

    user_id: Annotated[float, pydantic.Field(alias="userId")]

    auth_options: Annotated[
        Nullable[TemplateCreateDocumentFromTemplateAuthOptions],
        pydantic.Field(alias="authOptions"),
    ]

    form_values: Annotated[
        Nullable[Dict[str, TemplateCreateDocumentFromTemplateFormValues]],
        pydantic.Field(alias="formValues"),
    ]

    title: str

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]

    updated_at: Annotated[str, pydantic.Field(alias="updatedAt")]

    completed_at: Annotated[Nullable[str], pydantic.Field(alias="completedAt")]

    deleted_at: Annotated[Nullable[str], pydantic.Field(alias="deletedAt")]

    team_id: Annotated[float, pydantic.Field(alias="teamId")]

    folder_id: Annotated[Nullable[str], pydantic.Field(alias="folderId")]

    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]

    internal_version: Annotated[float, pydantic.Field(alias="internalVersion")]

    document_data: Annotated[
        TemplateCreateDocumentFromTemplateDocumentData,
        pydantic.Field(alias="documentData"),
    ]

    document_meta: Annotated[
        TemplateCreateDocumentFromTemplateDocumentMeta,
        pydantic.Field(alias="documentMeta"),
    ]

    envelope_items: Annotated[
        List[TemplateCreateDocumentFromTemplateEnvelopeItem],
        pydantic.Field(alias="envelopeItems"),
    ]

    folder: Nullable[TemplateCreateDocumentFromTemplateFolder]

    recipients: List[TemplateCreateDocumentFromTemplateRecipientResponse]

    fields: List[TemplateCreateDocumentFromTemplateField]

    template_id: Annotated[
        OptionalNullable[float], pydantic.Field(alias="templateId")
    ] = UNSET

    document_data_id: Annotated[
        Optional[str], pydantic.Field(alias="documentDataId")
    ] = ""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["templateId", "documentDataId"]
        nullable_fields = [
            "externalId",
            "authOptions",
            "formValues",
            "completedAt",
            "deletedAt",
            "folderId",
            "templateId",
            "folder",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m
